# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './wallaby/frontends/qt/isheet/gbMultipage.ui'
#
# Created: Mon Dec  3 18:07:06 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_CRoom(object):
    def setupUi(self, CRoom):
        CRoom.setObjectName(_fromUtf8("CRoom"))
        CRoom.resize(469, 158)
        self.verticalLayout = QtGui.QVBoxLayout(CRoom)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label = QtGui.QLabel(CRoom)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        self.LineEdit = LineEdit(CRoom)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.verticalLayout.addWidget(self.LineEdit)
        self.label_2 = QtGui.QLabel(CRoom)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout.addWidget(self.label_2)
        self.LineEdit_2 = LineEdit(CRoom)
        self.LineEdit_2.setObjectName(_fromUtf8("LineEdit_2"))
        self.verticalLayout.addWidget(self.LineEdit_2)

        self.retranslateUi(CRoom)
        QtCore.QMetaObject.connectSlotsByName(CRoom)

    def retranslateUi(self, CRoom):
        CRoom.setWindowTitle(QtGui.QApplication.translate("CRoom", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("CRoom", "Container\'s identifier", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("CRoom", "inspector-identifier", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("CRoom", "Default sheet name", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit_2.setProperty("template", QtGui.QApplication.translate("CRoom", "inspector-defaultsheet", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from wallaby.qt_combat import *

import gc, shutil

class GarbageCollector(QObject):

    '''
    Disable automatic garbage collection and instead collect manually
    every INTERVAL milliseconds.

    This is done to ensure that garbage collection only happens in the
    thread, as otherwise Qt can crash.
    '''

    INTERVAL = 5000

    def __init__(self, parent, debug=False):
        QObject.__init__(self, parent)
        self.debug = debug

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.check)

        self.threshold = gc.get_threshold()
        gc.disable()
        self.timer.start(self.INTERVAL)
        #gc.set_debug(gc.DEBUG_SAVEALL)

    def check(self):
        #return self.debug_cycles()
        l0, l1, l2 = gc.get_count()
        if self.debug:
            print ('gc_check called:', l0, l1, l2)
        if l0 > self.threshold[0]:
            num = gc.collect(0)
            if self.debug:
                print ('collecting gen 0, found:', num, 'unreachable')
            if l1 > self.threshold[1]:
                num = gc.collect(1)
                if self.debug:
                    print ('collecting gen 1, found:', num, 'unreachable')
                if l2 > self.threshold[2]:
                    num = gc.collect(2)
                    if self.debug:
                        print ('collecting gen 2, found:', num, 'unreachable')

    def debug_cycles(self):
        gc.collect()
        for obj in gc.garbage:
            print (obj, repr(obj), type(obj))





# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from wallaby.frontends.qt.widgets.drag.dragLogic import *

class LockLayer(QtGui.QWidget, DragLogic):
    Pixmap = None
    DragPixmap = None

    def __init__(self):
        QtGui.QWidget.__init__(self)
        DragLogic.__init__(self)

        if LockLayer.Pixmap == None:
            LockLayer.Pixmap = QtGui.QPixmap.fromImage(QtGui.QImage(':/icons/images/schloss.png').scaled(65, 65))

        self._pen = QtGui.QPen(QtGui.QColor(255, 0, 0, 128))

        if LockLayer.DragPixmap == None:
            LockLayer.DragPixmap = QtGui.QPixmap(':/icons/images/move.png')

    def pixmap(self):
        return LockLayer.DragPixmap

    def paintEvent(self, e):
        p = QtGui.QPainter(self)
        p.setBrush(QtGui.QColor(128,128,128, 128))
        p.setPen(QtCore.Qt.NoPen)
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)
        p.drawRoundedRect(0,0,self.width(), self.height(), 5,5) 
        p.setPen(self._pen)
        # p.drawText(0,0,self.width(), self.height(), QtCore.Qt.AlignCenter, "BLA")
        p.drawPixmap(self.width()/2-LockLayer.Pixmap.width()/2, self.height()/2-LockLayer.Pixmap.height()/2, LockLayer.Pixmap)

    def mouseDoubleClickEvent(self, e):
        self.parent().unlock()
        e.accept()

    def mousePressEvent(self, e):
        DragLogic.mousePressEvent(self, e)

    def mouseMoveEvent(self, e):
        DragLogic.mouseMoveEvent(self, e)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys
from wallaby.common.sets import OrderedSet

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.actionButtonPeer import *

class ActionButton(QtGui.QPushButton, BaseWidget, EnableLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    room     = Meta.property("string")
    pillow   = Meta.property("string")
    feathers = Meta.property("string")
    tab      = Meta.property("string")

    triggers = Meta.property("list", readOnly=True, default=["", "clicked", "pressed", "released"])

    def __init__(self, *args):
        QtGui.QPushButton.__init__(self, *args)

        BaseWidget.__init__(self, QtGui.QPushButton, *args)
        EnableLogic.__init__(self)
        TriggeredPillowsLogic.__init__(self)

        self._peer = None

        self.setEnabled(True)
        self.clicked.connect(self.buttonClicked)

        from functools import partial
        self.clicked.connect(partial(self.trigger, "clicked", None))
        self.pressed.connect(partial(self.trigger, "pressed"))
        self.released.connect(partial(self.trigger, "released"))

        self.setFocusPolicy(QtCore.Qt.StrongFocus)

    def overlayLabel(self):
        rooms = OrderedSet()
        rooms[self.room] = True

        if self.pillow != None:
            rooms |= House.pillowRooms(self.room, self.pillow, self.feathers)

        if len(rooms) == 0:
            rooms.add(None)

        label = []
        label.append(rooms)
        if self.pillow != None:
            label.append(self.pillow)

        return label

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

        if self._peer: self._peer.destroy(remove)
        self._peer = None

    def register(self):
        EnableLogic.register(self)
        TriggeredPillowsLogic.register(self)

        if self.pillow:
            self._peer = ActionButtonPeer(self.room, self.pillow, self.feathers, tab=self.tab, otherPillows=None)

    def buttonClicked(self):
        if self._peer: self._peer.buttonClicked()
        



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import sys
from thread import get_ident
import wallaby.FXUI as FXUI

from wallaby.qt_combat import *

class BaseQObject(QtCore.QObject):
    MAIN_THREAD_ID = 0
    
    def __init__(self):
        QtCore.QObject.__init__(self)
    #    self.installEventFilter(self)
        self.event = None

    # def eventFilter(self,obj,event):
    #   # FIXME:  This is a workaround for an unexplained bug
    #   # The events were getting posted through postEVentWithCallback()
    #   # But the event() method wasn't getting called.  But the eventFilter()
    #   # method is getting called.  
    #   if event.type()==QEvent.User:
    #       cb = event.__dict__.get('callback')
    #       if cb: 
    #           self._doEvent(event)
    #       return True
    #   return QObject.eventFilter(self,obj,event)

    # def _doEvent(self,event):
    #    cb = event.__dict__.get('callback')
    #    if not cb: return
    #    data = event.__dict__.get('data')
    #    if data or type(data)==type(False): cb(data)
    #    else: cb()
    #    del event

    def event(self, event):
        if event.type()==QtCore.QEvent.User:
            # self._doEvent(event)

            cb = event.__dict__.get('callback')
            if not cb: return
            data = event.__dict__.get('data')
            if data or type(data)==type(False): cb(data)
            else: cb()
            del event

            return True
        return QtCore.QObject.event(self, event)

    def postEventWithCallback(self, callback, data=None):
        # if we're in main thread, just fire off callback
        if get_ident()==BaseQObject.MAIN_THREAD_ID:
            if data or type(data)==type(False): callback(data)
            else: callback()
        # send callback to main thread 
        else:
            event = QtCore.QEvent(QtCore.QEvent.User)
            event.callback = callback
            if data or type(data)==type(False): event.data = data
            # FXUI.app.postEvent(self, event, -1000)
            FXUI.app.postEvent(self, event)
            
class Interleaver(BaseQObject):
    def quit(self):
        from twisted.internet import reactor
        reactor.stop()
        
    def __init__(self):
        BaseQObject.__init__(self)

    def toInterleave(self, func, *args, **kwargs):
        # FX.debug('toInterleave(): %s'%(str(func)))
        self.postEventWithCallback(func)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.frontends.qt.meta import *
from wallaby.qt_combat import *
from container import *
from wallaby.pf.peer.tab import *

class Stacked(QtGui.QStackedWidget, Container):
    __metaclass__ = QWallabyMeta

    identifier = Meta.property("string")
    defaultSheet = Meta.property("string")

    def __init__(self, *args):
        QtGui.QStackedWidget.__init__(self, *args)
        Container.__init__(self, QtGui.QStackedWidget, *args)

        self._peer = None
        self._sheetPrototypes = {}

        self.currentChanged.connect(self._tabChanged)

    def isMultiPage(self):
        return True

    def childCount(self):
        return self.count()

    def getChildWidget(self, pos):
        return self.widget(pos)

    def addChildWidget(self, w):
        self.addWidget(w)

    def removeChildWidget(self, w):
        self.removeWidget(w)
        w.deleteLater()

    def overlayRect(self):
        r = BaseWidget.overlayRect(self)
        r.moveTo(r.x(), 22)

        r.setHeight(18) 
        return r

    def _tabChanged(self, index):
        if index >= 0:
            containers = self.currentWidget().findChildren(Container)
            for c in containers: c.activate()

            if FX.mainWindow and FX.mainWindow.overlayMode():
                from twisted.internet import reactor
                reactor.callLater(0, FX.mainWindow.reloadOverlays)

    def doSetCurrentIndex(self, idx):
        if self.currentWidget() is not None:
            self.currentWidget().setSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Ignored)

        self.setCurrentIndex(idx)
        self.currentWidget().setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        self.adjustSize()

    def selectByIndex(self, idx):
        self.doSetCurrentIndex(idx)

    def selectByName(self, name):
        w = self.findChildren(QtGui.QWidget, name + "Sheet")
        if w != None and len(w) == 1: self.doSetCurrentIndex(self.indexOf(w[0]))
        elif self.defaultSheet != None:
            w = self.findChildren(QtGui.QWidget, self.defaultSheet + "Sheet")
            if w != None and len(w) == 1: self.doSetCurrentIndex(self.indexOf(w[0]))
            

    def deregister(self, remove=False):
        if self._peer: self._peer.destroy(remove)
        self._peer = None

    def register(self):
        Container.register(self)
        self._peer = Tab(self.room, self, self.identifier)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/iTextPopupButton.ui'
#
# Created: Sun Dec 16 18:13:24 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_ILineEdit(object):
    def setupUi(self, ILineEdit):
        ILineEdit.setObjectName(_fromUtf8("ILineEdit"))
        ILineEdit.resize(426, 307)
        self.verticalLayout = QtGui.QVBoxLayout(ILineEdit)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox = GroupBox(ILineEdit)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout.addWidget(self.groupBox)
        self.groupBox_4 = QtGui.QGroupBox(ILineEdit)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self.gridLayout = QtGui.QGridLayout(self.groupBox_4)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.LineEdit = LineEdit(self.groupBox_4)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.gridLayout.addWidget(self.LineEdit, 0, 1, 1, 1)
        self.label = QtGui.QLabel(self.groupBox_4)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_4)
        self.groupBox_3 = GroupBox(ILineEdit)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.verticalLayout.addWidget(self.groupBox_3)
        self.groupBox_2 = GroupBox(ILineEdit)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout.addWidget(self.groupBox_2)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(ILineEdit)
        QtCore.QMetaObject.connectSlotsByName(ILineEdit)

    def retranslateUi(self, ILineEdit):
        ILineEdit.setWindowTitle(QtGui.QApplication.translate("ILineEdit", "LineEdit preferences", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("ILineEdit", "Basic settings", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-groupbox-roomandpath", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_4.setTitle(QtGui.QApplication.translate("ILineEdit", "Popup settings", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-popuptitle", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("ILineEdit", "Title of popup", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("ILineEdit", "Triggered pillows", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-groupbox-triggeredpillows", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("ILineEdit", "Enable logic", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-groupbox-enablelogic", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit
from wallaby.frontends.qt.widgets.containers.groupBox import GroupBox



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from peer import *

from wallaby.qt_combat import *
from viewer import Viewer
from wallaby.backends.couchdb import *

class AttachmentOpener(Peer):
    Open = Pillow.In

    def __init__(self, room, databaseName=None):
        Peer.__init__(self, room)
        self._viewer = Viewer(room, self._setID, '_id')

        self._databaseName = databaseName
        self._id = None
        self._catch(AttachmentOpener.In.Open, self._open)

    def _setID(self, value):
        self._id = value

    def _open(self, pillow, feather):
        if self._id != None:
            dbURL = Database.getURLForDatabase(self._databaseName)
            if dbURL == None: return

            if self._databaseName == None:
                self._databaseName = Database.getDefaultDatabaseName()

            print dbURL, self._databaseName, self._id, feather

            url = QtCore.QUrl(dbURL+'/'+self._databaseName+'/'+self._id+'/'+feather)
            QtGui.QDesktopServices.openUrl(url)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/gbEnableLogic.ui'
#
# Created: Fri Dec 28 12:03:49 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(523, 403)
        Form.setMaximumSize(QtCore.QSize(16777215, 450))
        self.verticalLayout_2 = QtGui.QVBoxLayout(Form)
        self.verticalLayout_2.setMargin(0)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.groupBox = QtGui.QGroupBox(Form)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.checkBox = CheckBox(self.groupBox)
        self.checkBox.setObjectName(_fromUtf8("checkBox"))
        self.verticalLayout.addWidget(self.checkBox)
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        self.embeddedViewTable = EmbeddedViewTable(self.groupBox)
        self.embeddedViewTable.setMinimumSize(QtCore.QSize(0, 150))
        self.embeddedViewTable.setObjectName(_fromUtf8("embeddedViewTable"))
        self.verticalLayout.addWidget(self.embeddedViewTable)
        self.widget = QtGui.QWidget(self.groupBox)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.actionButton = ActionButton(self.widget)
        self.actionButton.setObjectName(_fromUtf8("actionButton"))
        self.horizontalLayout.addWidget(self.actionButton)
        self.actionButton_2 = ActionButton(self.widget)
        self.actionButton_2.setObjectName(_fromUtf8("actionButton_2"))
        self.horizontalLayout.addWidget(self.actionButton_2)
        self.verticalLayout.addWidget(self.widget)
        self.verticalLayout_2.addWidget(self.groupBox)
        self.groupBox_2 = QtGui.QGroupBox(Form)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.gridLayout = QtGui.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_2 = QtGui.QLabel(self.groupBox_2)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.LineEdit = LineEdit(self.groupBox_2)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.gridLayout.addWidget(self.LineEdit, 0, 1, 1, 1)
        self.label_3 = QtGui.QLabel(self.groupBox_2)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.LineEdit_2 = LineEdit(self.groupBox_2)
        self.LineEdit_2.setObjectName(_fromUtf8("LineEdit_2"))
        self.gridLayout.addWidget(self.LineEdit_2, 1, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.groupBox_2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("Form", "Payload-based enable triggers", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox.setText(QtGui.QApplication.translate("Form", "Hide the widget, when it is disabled", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox.setProperty("template", QtGui.QApplication.translate("Form", "inspector-hideIfDisabled", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "Enable the widget if one of the following feathers is found in an enable pillow.", None, QtGui.QApplication.UnicodeUTF8))
        self.embeddedViewTable.setProperty("template", QtGui.QApplication.translate("Form", "inspector-enable-feathers", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton.setText(QtGui.QApplication.translate("Form", "add", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton.setProperty("template", QtGui.QApplication.translate("Form", "inspector-enable-feathers-add", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton_2.setText(QtGui.QApplication.translate("Form", "remove", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton_2.setProperty("template", QtGui.QApplication.translate("Form", "inspector-enable-feathers-remove", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("Form", "Document-based enable triggers", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Form", "Path to the enable value", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("Form", "inspector-enable-path", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Form", "Enable value", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit_2.setProperty("template", QtGui.QApplication.translate("Form", "inspector-enable-value", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.buttons.checkBox import CheckBox
from wallaby.frontends.qt.widgets.multi.embeddedViewTable import EmbeddedViewTable
from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit
from wallaby.frontends.qt.widgets.buttons.actionButton import ActionButton



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from wallaby.qt_combat import *

from UI_mainWindow import Ui_MainWindow

from wallaby.pf.room import *
from wallaby.frontends.qt.baseWindow import *

import wallaby.backends.couchdb as couchdb
import wallaby.backends.elasticsearch as es

import app_rc

class MainWindow(BaseWindow, Ui_MainWindow):
    def __init__(self, quitCB, options):
        db = options.app
        if options.db is not None:
            db = options.db

        BaseWindow.__init__(self, "wallaby", options.app, options, quitCB, dbName=db)

        # set up User Interface (widgets, layout...)
        self.setupUi(self)

    def setConnectionSettings(self, options):
        if options and options.fx:
            options.server = "https://relax.freshx.de"
            options.couchPort = "443"
            options.esPort = "443/es"

        couch.Database.setURLForDatabase(self.dbName(), options.server + ":" + options.couchPort)
        es.Connection.setURLForIndex(None, options.server + ':' + options.esPort)

        if options and options.username != None and options.password != None:
            es.Connection.setLoginForIndex(None, options.username, options.password)

    def _credentialsArrived(self, pillow, feathers):
        pass



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
from wallaby.common.sets import OrderedSet

class WallabyPopup(QtGui.QDialog):
    def __init__(self, parent):
        QtGui.QWidget.__init__(self, parent)

        self.setWindowTitle("Wallaby - properties")

        flags = self.windowFlags()

        self.setMinimumSize(300, 200)
        self.setMaximumSize(300, 200)

        self.hide()

class WallabyOverlay(QtGui.QWidget):
    Pixmap = None
    Brushes = None

    RoomBrushes = {}
    NextBrush = 0

    @classmethod
    def resetColors(cls):
        cls.Brushes = None
        cls.NextBrush = 0
        cls.RoomBrushes = {}

    @classmethod
    def getColor(cls, room, width=None, height=None):
        if cls.Brushes == None:
            cls.Brushes = (
                ( QtGui.QColor(128,   0, 0  , 128), QtGui.QColor(128, 0  , 0  , 200) ), 
                ( QtGui.QColor(0  , 128, 0  , 128), QtGui.QColor(0  , 128, 0  , 200) ), 
                ( QtGui.QColor(0  , 0  , 128, 128), QtGui.QColor(0  , 0  , 128, 200) ), 
                ( QtGui.QColor(128, 128, 0  , 128), QtGui.QColor(128, 128, 0  , 200) ), 
                ( QtGui.QColor(0  , 128, 128, 128), QtGui.QColor(0  , 128, 128, 200) ), 
                ( QtGui.QColor(128, 0  , 128, 128), QtGui.QColor(128, 0  , 128, 200) ), 
                ( QtGui.QColor(192, 128, 64 , 128), QtGui.QColor(192, 128, 64 , 200) ), 
                ( QtGui.QColor(64 , 192, 128, 128), QtGui.QColor(64 , 192, 128, 200) ), 
                ( QtGui.QColor(128, 64 , 192, 128), QtGui.QColor(128, 64 , 192, 200) ), 
                ( QtGui.QColor(128, 128, 128, 128), QtGui.QColor(128, 128, 128, 200) )
            )

            cls.RoomBrushes[None] = 0
            cls.NextBrush = 1

        if width != None and isinstance(room, OrderedSet):
            cnt = len(room)
            if cnt >= 2:
                gradLL = QtGui.QLinearGradient(0, 0, width, height)
                gradHL = QtGui.QLinearGradient(0, 0, width, height)
                idx = 0.0
                cnt = float(cnt-1)
                for r in room:
                    gradLL.setColorAt(idx/cnt, cls.getColor(r)[0])
                    gradHL.setColorAt(idx/cnt, cls.getColor(r)[1])
                    idx += 1.0

                return (gradLL, gradHL)
            else:
                for r in room:  
                    room = r
                    break

        if isinstance(room, OrderedSet):
            return cls.Brushes[0]

        if room not in cls.RoomBrushes:
            cls.RoomBrushes[room] = cls.NextBrush
            cls.NextBrush += 1
            if cls.NextBrush >= len(cls.Brushes): cls.NextBrush = 1

        return cls.Brushes[cls.RoomBrushes[room]]

    def __init__(self, parent, host):
        QtGui.QWidget.__init__(self, parent)

        self._host = host

        if WallabyOverlay.Pixmap == None:
            WallabyOverlay.Pixmap = QtGui.QPixmap.fromImage(QtGui.QImage(':/images/images/wallaby.png').scaled(32, 32))

        self._hlPen = QtGui.QPen(QtGui.QColor(255, 255, 255, 222))
        self._llPen = QtGui.QPen(QtGui.QColor(255, 255, 255, 200))

        self._borderPen = QtGui.QPen(QtGui.QColor(0, 0, 0, 255))

        self._active = False
        self._showPixmap = False
        self._margin = 2
        self._label = ""
        self._room = None
        self._shortLabel = ""
        self._longLabel = ""
        self._baseGeometry = None
        self._anim = None
        self._lastWidth = None

        # self._popup = WallabyPopup(self)

    def host(self):
        return self._host

    def setBaseGeometry(self, geo):
        self._stopAnimation()

        self._baseGeometry = geo
        self.setGeometry(geo)

    def show(self):
        labels = self._host.overlayLabel()

        if len(labels) > 0:
            self._room = labels[0]
        else:
            self._room = None

        if len(labels) > 1: self._shortLabel = " | ".join(labels[1:])
        else: self._shortLabel = ""

        if self._room != None:
            if isinstance(self._room, OrderedSet):
                lst = self._room
                if None in lst: lst.remove(None)
                self._longLabel = "+".join(lst)
            else:
                self._longLabel = self._room
        else:   
            self._longLabel = "-"

        if self._shortLabel != "":
            self._longLabel += " | " + self._shortLabel
        else:
            self._shortLabel = self._longLabel

        self._label = self._shortLabel
        self._lastWidth = None

        self.setMouseTracking(True)
        QtGui.QWidget.show(self)
        self.repaint()

    def setBrushes(self):
        brushes = WallabyOverlay.getColor(self._room, self.width(), self.height())
        if self._active:
            self._pen = self._hlPen
            self._brush = brushes[1]
        else:
            self._pen = self._llPen
            self._brush = brushes[0]

    def hide(self):
        self.setMouseTracking(False)
        QtGui.QWidget.hide(self)

    def enterEvent(self, e):
        self._stopAnimation()

        fm = QtGui.QFontMetrics(self.font())

        self.raise_()
        self._active = True
        self.setBrushes()

        self._label = self._longLabel

        geo = self.geometry()

        w = geo.width()
        dw = fm.width(self._label) + 20 - w

        if dw > 0:
            w += dw
            geo.setWidth(w)

            offset = dw/2

            if self.pos().x() - offset + w > FX.mainWindow.width():
                offset = - (FX.mainWindow.width() - self.pos().x() - w)

            geo.moveTo(self.pos().x()-offset, self.pos().y())


        if geo != self.geometry():
            self._anim = QtCore.QPropertyAnimation(self ,"geometry");
            self._anim.setDuration(500)
            self._anim.setStartValue(self.geometry())
            self._anim.setEndValue(geo)
            self._anim.start()
        else:
            self.repaint()

    def _stopAnimation(self):
        if self._anim != None and self._anim.state() == QtCore.QAbstractAnimation.Running:
            self._anim.stop()

        self._anim = None

    def leaveEvent(self, e):
        self._stopAnimation()

        self._active = False
        self.setBrushes()

        self._label = self._shortLabel

        if self._baseGeometry != None and self.geometry() != self._baseGeometry:
            self._anim = QtCore.QPropertyAnimation(self ,"geometry");
            self._anim.setDuration(300)
            self._anim.setStartValue(self.geometry())
            self._anim.setEndValue(self._baseGeometry)
            self._anim.start()
        else:
            self.repaint()

    def paintEvent(self, e):
        if self.width() != self._lastWidth:
            self.setBrushes()

        self._lastWidth = self.width()

        p = QtGui.QPainter(self)
        p.setBrush(self._brush)
        p.setPen(self._borderPen)
            
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)
        p.drawRoundedRect(0,0,self.width(), self.height(), 5,5) 
        p.setPen(self._pen)
        label = self._label

        x = self._margin
        y = self._margin  
        if self._showPixmap: 
            x += self._margin + WallabyOverlay.Pixmap.width()

        p.drawText(x, y, self.width()-x-self._margin, self.height()-y-self._margin, QtCore.Qt.AlignCenter, label)

        if self._showPixmap:
            p.drawPixmap(self._margin, self._margin, WallabyOverlay.Pixmap)

    def mousePressEvent(self, e):
        FX.mainWindow.openConfig(self._host)
        e.accept()

    def mouseDoubleClickEvent(self, e):
        e.ignore()



#!/usr/bin/env python
# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.


import wallaby.FXUI as FXUI

from wallaby.qt_combat import *

from UI_loginDialog import Ui_LoginDialog

class LoginDialog(QtGui.QDialog, Ui_LoginDialog):

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)

        # set up User Interface (widgets, layout...)
        self.setupUi(self)



#!/usr/bin/env python
# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.


from __future__ import with_statement
from __future__ import division

_TRY_PYSIDE = False

try:
    if not _TRY_PYSIDE:
        raise ImportError()
    import PySide.QtCore as _QtCore
    import PySide.QtGui as _QtGui
    import PySide.QtWebKit as _QtWebKit
    import PySide.QtDesigner as _QtDesigner
    QtCore = _QtCore
    QtGui = _QtGui
    QtWebKit = _QtWebKit
    QtDesigner = _QtDesigner
    USES_PYSIDE = True
except ImportError:
    import sip
    sip.setapi('QString', 2)
    sip.setapi('QVariant', 2)
    import PyQt4.QtCore as _QtCore
    import PyQt4.QtGui as _QtGui
    import PyQt4.QtWebKit as _QtWebKit
    import PyQt4.QtDesigner as _QtDesigner
    QtCore = _QtCore
    QtGui = _QtGui
    QtWebKit = _QtWebKit
    QtDesigner = _QtDesigner
    USES_PYSIDE = False


def _pyside_import_module(moduleName):
    pyside = __import__('PySide', globals(), locals(), [moduleName], -1)
    return getattr(pyside, moduleName)


def _pyqt4_import_module(moduleName):
    pyside = __import__('PyQt4', globals(), locals(), [moduleName], -1)
    return getattr(pyside, moduleName)


if USES_PYSIDE:
    import_module = _pyside_import_module

    Signal = QtCore.Signal
    Slot = QtCore.Slot
    Property = QtCore.Property
else:
    import_module = _pyqt4_import_module

    Signal = QtCore.pyqtSignal
    Slot = QtCore.pyqtSlot
    Property = QtCore.pyqtProperty


if __name__ == "__main__":
    pass



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from wallaby.frontends.qt.widgets.buttons.pushButton import PushButton
from ..logics import *

class ToggleButton(PushButton):
    __metaclass__ = QWallabyMeta

    def __init__(self, *args):
        PushButton.__init__(self, *args)
        self.setCheckable(True)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import json
from wallaby.common.draggable import *

class DragLogic(object):
    def __init__(self):
        self._dragStartPosition = None

    def mousePressEvent(self, event):
        if event.button() & QtCore.Qt.LeftButton:
            self._dragStartPosition = event.pos()

    def mouseMoveEvent(self, event):
        if not event.buttons() & QtCore.Qt.LeftButton: return
        if (event.pos() - self._dragStartPosition).manhattanLength() < QtGui.QApplication.startDragDistance(): return

        p = self.parent()
        while not isinstance(p, Draggable) and not p is None:
            p = p.parent()
        if p is None:
            return

        drag = QtGui.QDrag(self)

        mimeData = QtCore.QMimeData()
        mimeData.setText(p.dragURL())
        # mimeData.setUrls([QtCore.QUrl(p.dragURL())])
        p.dragCustom(mimeData)

        drag.setPixmap(self.pixmap())
        drag.setMimeData(mimeData)
        drag.exec_(QtCore.Qt.CopyAction | QtCore.Qt.MoveAction, QtCore.Qt.MoveAction)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FXUI as FXUI

from wallaby.frontends.qt.meta import *
from wallaby.qt_combat import *
from wallaby.pf.peer.tab import *

from container import *

class Tabs(QtGui.QTabWidget, Container):
    __metaclass__ = QWallabyMeta

    identifier = Meta.property("string")
    defaultSheet = Meta.property("string")

    def __init__(self, *args):
        QtGui.QTabWidget.__init__(self, *args)
        Container.__init__(self, *args)

        self._peer = None
        self.currentChanged.connect(self._tabChanged)

    def isMultiPage(self):
        return True

    def overlayRect(self):
        r = BaseWidget.overlayRect(self)
        if self.documentMode():
            r.moveTo(r.x(), self.height())
        else:
            r.moveTo(r.x(), 22)

        r.setHeight(18) 
        return r

    def childCount(self):
        return self.count()

    def addChildWidget(self, w):
        self.addTab(w, w.windowTitle())

    def getChildWidget(self, pos):
        return self.widget(pos)

    def removeChildWidget(self, pos):
        self.removeTab(pos)

    def _tabChanged(self, index):
        containers = self.currentWidget().findChildren(Container)
        for c in containers: c.activate()

        if FXUI.mainWindow and FXUI.mainWindow.overlayMode():
            from twisted.internet import reactor
            reactor.callLater(0, FXUI.mainWindow.reloadOverlays)

    def deregister(self, remove=False):
        if self._peer: self._peer.destroy(remove)
        self._peer = None

    def selectByIndex(self, idx):
        self.setCurrentIndex(idx)

    def selectByName(self, name):
        w = self.findChildren(QtGui.QWidget, name + "Sheet")
        if w != None and len(w) == 1: self.setCurrentWidget(w[0])
        elif self.defaultSheet != None:
            w = self.findChildren(QtGui.QWidget, self.defaultSheet + "Sheet")
            if w != None and len(w) == 1: self.setCurrentWidget(w[0])

    def register(self):
        Container.register(self)
        self._peer = Tab(self.room, self, self.identifier)




#!/usr/bin/env python
# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

from functools import partial

from wallaby.qt_combat import *

from wallaby.pf.peer.multiViewer import *

class TreeViewTableModel(QtCore.QAbstractItemModel):
    def __init__(self, widget, cols=['_id'], *args, **ka):
        QtCore.QAbstractTableModel.__init__(self, None)

        self._widget = widget

        self._peer = MultiViewer(*args, delegate=self, **ka)

        self._columns  = []
        self._types    = []
        self._typeArgs = []
        self._labels   = []
        self._cache    = []
        self._root     = QtCore.QModelIndex()

        self._columns, self._labels = FX.splitList(cols, ':')
        self._types, self._typeArgs, self._columns = FX.extractType(self._columns)

    def doMultiSelect(self, *args, **ka):
        # self._peer.doMultiSelect(*args, **ka)
        pass

    def doSelect(self, idx, tab):
        # self._peer.doSelect(idx, tab)
        pass

    def sort(self, col, order):
        # if not self._peer or col >= len(self._columns): return

        # if col == -1:
        #     self._peer.sort('__default__', order)
        # else:
        #     self._peer.sort(self._columns[col], order)
        pass

    def hasChildren(self, index):
        return False

    def beginInsertCB(self, f, cnt):
        self.beginInsertRows(self._parent, f, f + cnt - 1)

        i = f
        for j in range(cnt):
            cols = []
            for k in range(len(self._columns)): cols.append({'cached': False, 'value': None})

            self._cache.insert(i, {'cache': cols, 'loading': False})
            i = i + 1

    def resetSelectionCB(self):
        if self._widget: self._widget.resetSelection()

    def endInsertCB(self, f, cnt):
        self.emit(QtCore.SIGNAL('rowsInserted'), self._parent, f, cnt - 1)
        self.endInsertRows()

    def beginDeleteCB(self, f, t):
        self.beginRemoveRows(self._parent, f, t-1)

        for i in range(f, t):
            if f < len(self._cache):
                self._cache.pop(f)
            else:
                print "Try to remove out of bounds!", f, "of", len(self._cache) 

    def endDeleteCB(self, f, t):
        self.emit(QtCore.SIGNAL('rowsRemoved'), self._parent, f, t-1)
        self.endRemoveRows()

    def updateCB(self, row):
        if row < len(self._cache):
            self._cache[row]['loading'] = False
            for i in range(len(self._cache[row]['cache'])): self._cache[row]['cache'][i]['cached'] = False
            self.dataChanged.emit(self.index(row, 0), self.index(row, len(self._columns)-1))
                
    def parent(self, index):
        return self._parent

    def index(self, row, col, parent=QtCore.QModelIndex()):
        return self.createIndex(row, col)

    def rowCount(self, parent): 
        return len(self._cache)

    def columnCount(self, parent): 
        return len(self._columns)

    def getID(self, row):
        if len(self._rows) <= row:
            return "N/A"
        else:
            return self._rows[row].id

    def _empty(self):
        if FX.PyQt4:
            return QtCore.QVariant()
        else:
            return None

    def data(self, index, role): 
        if not index.isValid(): return self._empty()

        row = index.row()
        col = index.column()

        if role == QtCore.Qt.SizeHintRole:
            if len(self._columns) <= col or self.rowCount(self.parent) <= row or not self._cache[row]['cache'][col]['cached']:
                return self._empty()

            val = self._cache[row]['cache'][col]['value']

            if isinstance(val, QtGui.QPixmap):
                return val.size()
            else:
                return None
 
        if role != QtCore.Qt.DisplayRole or self.rowCount(self._parent) <= row or len(self._columns) <= col: return self._empty()

        # if self._cache[row]['loading']:
        #     if col == 0:
        #         return "Loading..."
        #     else:
        #         return ""

        if self._cache[row]['cache'][col]['cached']:
            val = self._cache[row]['cache'][col]['value']
            if val == None:
                return '-'
            else:
                return val

        self._cache[row]['loading'] = True
        d = self._peer.deferredGetValue(row, self._columns[col])
        d.addCallback(partial(self._updateField, row, col))

        if col == 0:
            return "Loading..."
        else:
            return ""

    def _updateField(self, row, col, val):
        if row >= len(self._cache): return

        val = FX.renderType(val, self._types[col], self._typeArgs[col]) 

        self._cache[row]['loading'] = False
        self._cache[row]['cache'][col]['cached'] = True
        self._cache[row]['cache'][col]['value']  = val

        self.dataChanged.emit(self.index(row, col), self.index(row, col))
        return val

    def headerData(self, col, orientation, role):
        if not QtCore: return
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            if col >= len(self._labels):
                if FX.PyQt4:
                    return QtCore.QVariant()
                else:
                    return None

            if FX.PyQt4:
                return QtCore.QVariant(self._labels[col])
            else:
                return unicode(self._labels[col])

        if orientation == QtCore.Qt.Vertical and role == QtCore.Qt.DisplayRole:
            if FX.PyQt4:
                return QtCore.QVariant('%03d' % (col+1))
            else:
                return unicode('%03d' % (col+1))

        if FX.PyQt4:
            return QtCore.QVariant()
        else:
            return None




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.listPeer import *
from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.multiViewer import *
from wallaby.pf.peer.editor import *
from twisted.internet import defer

class ComboBox(QtGui.QComboBox, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    isEditor = Meta.property("bool")
    source = Meta.property("string")
    sourcePath = Meta.property("string")

    selectPillow = Meta.property("string", extended=True)
    insertPillow = Meta.property("string", extended=True)

    isView = Meta.property("bool", extended=True)
    view           = Meta.property("string", extended=True)
    viewArguments  = Meta.property("dict", extended=True)
    viewIdentifier = Meta.property("string", extended=True)
    dataView       = Meta.property("bool", extended=True)

    viewPath = Meta.property("string", extended=True)
    viewDisplayPath = Meta.property("string", extended=True)
    throwWhileEditing = Meta.property("bool", extended=True)

    def __init__(self, *args):
        QtGui.QComboBox.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QComboBox, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self._setItemText)
        EditLogic.__init__(self, Editor, self._currentText)

        self.sourceVal = None
        self._changed = True
        self._lastValue = None

        self._subPath = None

        self._listPeer = None
        self._viewPeer = None
        self._multiViewer = None
        self._translate = {}
        self._translateReverse = {}
        self.currentIndexChanged.connect(self._selectOther)

    def _currentText(self):
        text = unicode(self.currentText())
        if text in self._translate:
            text = self._translate[text]

        return text

    def overlayLabel(self):
        label = BaseWidget.overlayLabel(self)
        if self.isView:
            label.append(self.view)
        else:
            path = self.sourcePath
            if path == None: path = 'titles'
            if self.source != None:
                label.append(self.source + ":" + path)
            else:
                label.append(path)
        
        return label

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        if self.lineEdit() != None: EditLogic.deregister(self, remove)

        if self._listPeer: self._listPeer.destroy(remove)
        self._listPeer = None

        if self._viewPeer: self._viewPeer.destroy(remove)
        self._viewPeer = None

        if self._multiViewer: self._multiViewer.destroy(remove)
        self._multiViewer = None

    def _textChanged(self, *args):
        if self.throwWhileEditing:
            self._editingFinished()

    def _editingFinished(self):
        if self._editor != None and self.lineEdit() != None and self._changed:
            self._editor._fieldChanged()
        if self.insertPillow != None:
            House.get(self.room).throw(self.insertPillow, unicode(self.lineEdit().text()))

    def changedCB(self, docID):
        # print "CHANGED:", docID
        pass

    def dataCB(self, result):
        from twisted.internet import reactor
        reactor.callLater(0, self._dataCB, result)

    @defer.inlineCallbacks
    def _dataCB(self, result):
        self._changed = False
        if result == None:
            self.clear()
            return

        viewDisplayPath = self.viewDisplayPath
        viewPath = self.viewPath
        if viewDisplayPath == None: viewDisplayPath = self.viewPath
        if viewDisplayPath == None: viewDisplayPath = "value"

        self._translate = {}
        self._translateReverse = {}

        if result.length() != self.count():
            self.clear()
            for i in range(result.length()):
                displayValue = yield result.deferredGetValue(i, viewDisplayPath)
                value = yield result.deferredGetValue(i, viewPath)
                if value != None:
                    self._translate[displayValue] = value
                    self._translateReverse[value] = displayValue
                    self.insertItem(i, displayValue)
        else:
            for i in range(result.length()):
                displayValue = yield result.deferredGetValue(i, viewDisplayPath)
                value = yield result.deferredGetValue(i, viewPath)
                if value != None:
                    self._translate[displayValue] = value
                    self._translateReverse[value] = displayValue
                    self.setItemText(i, displayValue)

        self._changed = True
        self._viewer._refresh(None, self.path) # Select path value

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        if self.isEditor != None: EditLogic.register(self, raw=True)

        if not self._registered:
            self._registered = True

            if self.lineEdit() != None:
                self.lineEdit().textChanged.connect(self._textChanged)
                self.lineEdit().editingFinished.connect(self._editingFinished)

        if self.isView:
            self._multiViewer = MultiViewer(self.room, self.view, self.viewIdentifier, self.viewArguments, self.dataView, self, autoUpdate=True)
        elif self.source != None:
            if self.sourcePath != None:
                if ':' in self.source:
                    room, source = self.source.split(':') 
                else:
                    room = self.room
                    source = self.source

                if '|' in self.sourcePath:
                    sourcePath, self._subPath = self.sourcePath.split('|')
                else:
                    sourcePath = self.sourcePath
                    self._subPath = None

                if source == "__DOC__":
                    self._listPeer = Viewer(room, self._fillComboBox, sourcePath, raw=True)
                else:
                    self._listPeer = ListPeer(room, self._fillComboBox, sourcePath, source)
        else:
            if self.sourcePath != None:
                if '|' in self.sourcePath:
                    sourcePath, self._subPath = self.sourcePath.split('|')
                else:
                    sourcePath = self.sourcePath
                    self._subPath = None

                self._viewPeer = Viewer(self.room, self._fillComboBox, self.sourcePath)

    # def mousePressEvent(self, event):
    #     QtGui.QComboBox.mousePressEvent(self, event)

    #     if self.isView:
    #         self._multiViewer._updateQuery()

    def _resolve(self, value, **ka):
        self._setItemText(value)
        if self._editor: 
            self._editor._resolve(**ka)
            self._selectOther(None)

    def _selectOther(self, val):
        if self.selectPillow != None and val >= 0:
            val = unicode(self.itemText(val))
            if val in self._translate: val = self._translate[val]
            # print val, isinstance(val, (list, tuple))
            House.get(self.room).throw(self.selectPillow, val)

        if self._editor and self._changed:
            self._editor._fieldChanged()

    def _setItemText(self, val):
        self._changed = False
        self._lastValue = val
        if val:
            val = unicode(val)
            self._lastValue = val # unicode version

            if val in self._translateReverse: val = self._translateReverse[val]
            exists = False
            self.setCurrentIndex(0)
            for i in range(self.count()):
                if val == unicode(self.itemText(i)):
                    self.setCurrentIndex(i)
                    exists = True
            if not exists:
                self.setCurrentIndex(0)
                if self.lineEdit() != None:
                    self.lineEdit().setText(unicode(val))
        else:
            self.setCurrentIndex(0)
            if self.lineEdit() != None:
                self.lineEdit().setText(u"")

        self._changed = True

    def _fillComboBox(self, document):
        self._changed = False

        self._translate = {}
        self._translateReverse = {}
        self.clear()

        if self.source != None and "__DOC__" in self.source:
            if document is not None and self._subPath != None:
                from wallaby.common.pathHelper import PathHelper
                titles = []
                for dct in document:
                    titles.append(PathHelper.getValue(dct, self._subPath))
            else:
                titles = document
        else:
            path = self.sourcePath
            if path == None: path = 'titles'
            titles = document.get(path)

        if titles:
            for i in range(len(titles)):
                value = titles[i]
                if isinstance(value, (list, tuple)):
                    displayValue, value = value
                else:
                    displayValue = value

                self._translate[displayValue] = value
                # if isinstance(value, (unicode, str)):
                self._translateReverse[unicode(value)] = displayValue

                self.insertItem(i, displayValue)
                if value == self._lastValue:
                    self.setCurrentIndex(i)

        self._changed = True



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FXUI as FXUI
from wallaby.qt_combat import *
from wallaby.frontends.qt.widgets.prefs.controller import PrefController

#============================================================================#
# The group name in designer widgetbox                                       #
#----------------------------------------------------------------------------#
DESIGNER_GROUP_NAME = "wallaby@fX"

#============================================================================#
# Plugin for CTLiveView                                                      #
#----------------------------------------------------------------------------#
class PrefPlugin(QtDesigner.QPyDesignerCustomWidgetPlugin):

    def __init__(self, parent=None):
        super(PrefPlugin, self).__init__(parent)

        self.initialized = False

    def initialize(self, formEditor):
        if self.initialized:
            return
        self.initialized = True

    def isInitialized(self):
        return self.initialized

    def isContainer(self):
        return False

    def icon(self):
        return FXUI.icon()

    def domXml(self):
        return '<widget class="PrefController" name="preferences">\n</widget>\n'
    
    def group(self):
        return DESIGNER_GROUP_NAME
              
    def includeFile(self):
        return "wallaby.frontends.qt.widgets.prefs.controller"

    def name(self):
        return "PrefController"

    def toolTip(self):
        return ""

    def whatsThis(self):
        return ""

    def createWidget(self, parent):
        return PrefController(parent)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
from container import *

class Widget(QtGui.QWidget, Container):
    __metaclass__ = QWallabyMeta

    def __init__(self, *args):
        QtGui.QWidget.__init__(self, *args)
        Container.__init__(self, QtGui.QWidget, *args)

        self._sheetPrototypes = {}
        self._layout = None

    def createLayout(self):
        self._layout = QtGui.QVBoxLayout()
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(6)
        self.setLayout(self._layout)

    def childCount(self):
        if self._layout is None: return 0
        return self._layout.count()

    def getChildWidget(self, pos):
        if self._layout is None: return None
        return self._layout.takeAt(pos).widget()

    def addChildWidget(self, w):
        if self._layout is None: self.createLayout()
        self._layout.addWidget(w)

    def removeChildWidget(self, w):
        Container.removeChildWidget(self, w)

    def overlayRect(self):
        r = BaseWidget.overlayRect(self)
        r.moveTo(r.x(), 22)

        r.setHeight(18) 
        return r

    def deregister(self, remove=False):
        pass

    def register(self):
        Container.register(self)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

class Layer(QtGui.QWidget):
    def __init__(self, name, *args):
        QtGui.QWidget.__init__(self, *args)

        self._name = name
        self._displayingContent = False
        self._userinteractionEnabled = True
        self._group = None
    

    def setDisplayingContent(self, displayingContent):
        self._displayingContent = displayingContent

    def setUserinteractionEnabled(self, userinteractionEnabled):
        self._userinteractionEnabled = userinteractionEnabled

    def userinteractionEnabled(self):
        return self._displayingContent == True and self._userinteractionEnabled == True

    def register(self):
        pass

    def deregister(self, remove=False):
        pass

    def paintEvent(self, event):
        if self._displayingContent == True:
            self.layerPaintEvent(event)

    def layerPaintEvent(self, event):
        pass

    def keyPressEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerKeyPressEvent(event)
        else:
            event.ignore()

    def layerKeyPressEvent(self, event):
        self.parentWidget().keyPressEvent(event)

    def keyReleaseEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerKeyReleaseEvent(event)
        else:
            event.ignore()

    def layerKeyReleaseEvent(self, event):
        self.parentWidget().keyReleaseEvent(event)

    def mouseMoveEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerMouseMoveEvent(event)
        else:
            event.ignore()

    def layerMouseMoveEvent(self, event):
        self.parentWidget().mouseMoveEvent(event)

    def mousePressEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerMousePressEvent(event)
        else:
            event.ignore()

    def layerMousePressEvent(self, event):
        self.parentWidget().mousePressEvent(event)

    def mouseDoubleClickEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerMouseDoubleClickEvent(event)
        else:
            event.ignore()

    def layerMouseDoubleClickEvent(self, event):
        self.parentWidget().mouseDoubleClickEvent(event)

    def mouseReleaseEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerMouseReleaseEvent(event)
        else:
            event.ignore()

    def layerMouseReleaseEvent(self, event):
        event.ignore()

    def wheelEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerWheelEvent(event)
        else:
            event.ignore()

    def layerWheelEvent(self, event):
        self.parentWidget().wheelEvent(event)

    def hoverEvent(self, event):
        if self.userinteractionEnabled() == True:
            self.layerHoverEvent(event)
        else:
            event.ignore()

    def layerHoverEvent(self, event):
        self.parentWidget().hoverEvent(event)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/iStacked.ui'
#
# Created: Sun Dec 16 18:13:23 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(400, 300)
        self.verticalLayout = QtGui.QVBoxLayout(Form)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox_2 = GroupBox(Form)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox = GroupBox(Form)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout.addWidget(self.groupBox)
        self.groupBox_3 = GroupBox(Form)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.verticalLayout.addWidget(self.groupBox_3)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("Form", "Basic settings", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setProperty("template", QtGui.QApplication.translate("Form", "inspector-groupbox-room", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("Form", "Sheets", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setProperty("template", QtGui.QApplication.translate("Form", "inspector-container-sheets", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("Form", "Multipage settings", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setProperty("template", QtGui.QApplication.translate("Form", "inspector-groupbox-multipage", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.containers.groupBox import GroupBox



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from wallaby.pf.peer.enabler import *
from wallaby.pf.peer.documentPathEnabler import *
from wallaby.pf.peer.actionButtonPeer import *
from wallaby.pf.peer.triggeredPillows import *
from wallaby.frontends.qt.meta import *

class WidgetNotSubclassingEnableLogicError(Exception):
    pass

class ContextMenuLogic(object):
    __metaclass__ = QWallabyMeta

    contextMenuEntries  = Meta.property("dict")

    def getMenuEntries(self):
        return self.contextMenuEntries

class TriggeredPillowsLogic(object):
    __metaclass__ = QWallabyMeta

    triggeredPillows = Meta.property("dict")

    def __init__(self):
        self._triggeredPillowsPeer = None
        self._triggeredViewer = None
        self._triggeredValue = None
        self._triggeredPath = None
        self._triggeredDoc = None

    def trigger(self, trigger, arg=None):
        if self._triggeredPillowsPeer != None: return self._triggeredPillowsPeer.trigger(trigger, arg, doc=self._triggeredDoc, path=self._triggeredPath, value=self._triggeredValue)
        return False

    def _newTriggeredValue(self, value):
        self._triggeredDoc = self._triggeredViewer.document()
        self._triggeredValue = value

    def register(self):
        try:
            if self.path != None and len(self.path) > 0:
                self._triggeredPath = self.path
        except: pass 

        if len(self.triggeredPillows) > 0:
            self._triggeredPillowsPeer = TriggeredPillows(self.room, self.triggeredPillows)

            if self._triggeredPath is not None: 
                self._triggeredViewer = Viewer(self.room, self._newTriggeredValue, self.path, raw=True)

    def deregister(self, remove=False):
        if self._triggeredViewer: self._triggeredViewer.destroy(remove)
        if self._triggeredPillowsPeer: self._triggeredPillowsPeer.destroy(remove)       
        self._triggeredViewer = None
        self._triggeredPillowsPeer = None

class EnableLogic(object):
    __metaclass__ = QWallabyMeta

    room            = Meta.property("string")
    enableFeathers  = Meta.property("list")
    enablePath      = Meta.property("string")
    enablePathValue = Meta.property("string")
    hideIfDisabled  = Meta.property("bool")

    def __init__(self):
        self._enabler = None
        self._documentPathEnabler = None

        self._enabledByEnabler = True
        self._enabledByEditor = True
        self._enabledByRealtime = True

    def deregister(self, remove=False):
        if self._enabler: self._enabler.destroy(remove)
        if self._documentPathEnabler: self._documentPathEnabler.destroy(remove)

        self._enabler = None
        self._documentPathEnabler = None

    def register(self):
        if len(self.enableFeathers) > 0:
            self._enabler = Enabler(self.room, self._enablerSetEnabled, self.enableFeathers)

        if self.enablePath != None and self.enablePathValue != None and self.enablePath != "":
            self._documentPathEnabler = DocumentPathEnabler(self.room, self._enablerSetEnabled, self.enablePath, self.enablePathValue)

    def _editorSetDisabled(self, disabled):
        self._enabledByEditor = not disabled
        self._callSetEnabled()

    def _enablerSetEnabled(self, enabled):
        self._enabledByEnabler = enabled
        self._callSetEnabled()

    def _realtimeSetEnabled(self, enabled):
        self._enabledByRealtime = enabled
        self._callSetEnabled()

    def _callSetEnabled(self):
        enabled = self._enabledByEnabler and self._enabledByEditor and self._enabledByRealtime
        self._setEnabled(enabled)

    def _setEnabled(self, enabled):
        if self.hideIfDisabled:
            self.setVisible(enabled)
        else:
            self.setEnabled(enabled)

class QueryLogic(object):
    __metaclass__ = QWallabyMeta

    view = Meta.property("string")
    viewArguments = Meta.property("string")
    viewIdentifier = Meta.property("string")
    dataView = Meta.property("string")

    def __init__(self):
        self._queryPeer = None

    def register(self, **viewerArgs):
        from wallaby.pf.peer.multiViewer import MultiViewer
        self._queryPeer = MultiViewer(self.room, self.view, self.viewIdentifier, self.viewArguments, self.dataView, self, autoUpdate=True)

    def initialData(self): 
        pass

    def deregister(self, remove=False):
        if self._queryPeer: self._queryPeer.destroy(remove)
        self._queryPeer = None

class ViewLogic(object):
    __metaclass__ = QWallabyMeta

    room = Meta.property("string")
    path = Meta.property("string")

    def __init__(self, viewerClass, setContentCB):
        self._viewerClass = viewerClass
        self._setContentCB = setContentCB
        self._viewer = None

    def register(self, **viewerArgs):
        if self.path and self._setContentCB: self._viewer = self._viewerClass(self.room, self._setContentCB, self.path, **viewerArgs)

    def deregister(self, remove=False):
        if self._viewer: self._viewer.destroy(remove)
        self._viewer = None

class EditLogic(object):
    __metaclass__ = QWallabyMeta

    room         = Meta.property("string")
    path         = Meta.property("string")
    selectOnEdit = Meta.property("bool")

    def __init__(self, editorClass, getContentCB):
        self._editorClass = editorClass
        self._getContentCB = getContentCB
        self._editor = None

    def register(self, **editorArgs):
        if not isinstance(self, EnableLogic): raise WidgetNotSubclassingEnableLogicError()

        if self.path and self._getContentCB: self._editor = self._editorClass(self.room, None, self._getContentCB, self._editorSetDisabled, self._conflict, self.path, selectOnEdit=self.selectOnEdit, **editorArgs)

    def deregister(self, remove=False):
        if self._editor: self._editor.destroy(remove)
        self._editor = None

class EnterPillowLogic(object):
    __metaclass__ = QWallabyMeta

    enterPillow   = Meta.property("string")
    enterFeathers = Meta.property("string")
    enterTab      = Meta.property("string")

    def __init__(self, signal):
        self._enterPeer = None
        signal.connect(self._returnPressed)

    def register(self):
        if self.enterPillow != None:
            self._enterPeer = ActionButtonPeer(self.room, self.enterPillow, self.enterFeathers, tab=self.enterTab)

    def deregister(self, remove=False):
        if self._enterPeer: self._enterPeer.destroy(remove)
        self._enterPeer = None

    def _returnPressed(self):
        if self._enterPeer:
            self._enterPeer.buttonClicked()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from wallaby.pf.peer.pref import *
from twisted.internet import defer
from UI_prefController import *
from sheet import *
from ..logics import *

class PrefController(QtGui.QWidget, Ui_PrefController, BaseWidget, EnableLogic):
    __metaclass__ = QWallabyMeta

    room = Meta.property("string")
    path = Meta.property("string")

    configDocId = Meta.property("string")

    def __init__(self, *args):
        QtGui.QWidget.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QWidget, *args)
        EnableLogic.__init__(self)
        self._pref = None
        self._viewer = None
        self._idx = 0
        self._sheetPrototypes = {}
        self._idxToPreference = []

        self.setupUi(self)

        self.scrollArea.setWidgetResizable(True)
        self._layout = QtGui.QVBoxLayout(self.content)

        self.prefAdd.clicked.connect(self.addPrefWidget)

        self._loading = False
        self._reload = False

        # self.setEnabled(False)

    def checkEnableUpDowns(self):
        for i in range(self._layout.count()):
            item = self._layout.itemAt(i)
            item.widget().moveUp.setEnabled(True)
            item.widget().moveDown.setEnabled(True)

        if self._layout.count() < 1: return
        self._layout.itemAt(0).widget().moveUp.setEnabled(False)
        self._layout.itemAt(self._layout.count()-1).widget().moveDown.setEnabled(False)

    def overlayRect(self):
        r = BaseWidget.overlayRect(self)
        r.moveTo(r.x(), r.height()-60)
        r.setHeight(min(20, r.height()))
        return r

    def movePrefSheet(self, pos, direction):
        doc = self._pref.document()
        if doc == None: return

        list = doc.get(self.path)
        if pos > len(list): return

        if direction and pos == 0: return
        if not direction and pos == len(list)-1: return

        if direction:
            list[pos-1], list[pos] = list[pos], list[pos-1]

            item = self._layout.takeAt(pos)
            self._layout.insertWidget(pos-1, item.widget())

            item.widget().decrementIndex()
            self._layout.itemAt(pos).widget().incrementIndex()
        else:
            list[pos+1], list[pos] = list[pos], list[pos+1]

            item = self._layout.takeAt(pos)
            self._layout.insertWidget(pos+1, item.widget())

            item.widget().incrementIndex()
            self._layout.itemAt(pos).widget().decrementIndex()

        self.checkEnableUpDowns()

    def move(self, f, t):
        if f < t: t-=1
        if f==t: return

        doc = self._pref.document()
        if doc == None: return

        list = doc.get(self.path)
        if t > len(list): return

        data = list[f]
        del list[f]
        list.insert(t, data)

        item = self._layout.takeAt(f)

        self._layout.insertWidget(t, item.widget())

        for i in range(self._layout.count()):
            item = self._layout.itemAt(i)
            item.widget().setIndex(i)

        self.checkEnableUpDowns()

        self._pref._throw(Editor.Out.FieldChanged, self.path + ".")

    def addPrefWidget(self):
        if len(self._idxToPreference) == 0 or self.prefSelect.count() == 0: return

        sheet = self._idxToPreference[self.prefSelect.currentIndex()]
        from twisted.internet import reactor
        if sheet is not None:
            print "Create sheet manual"
            self.createSheet(unicode(sheet), locked=False, scrollTo=True)

    def register(self):
        EnableLogic.register(self)
        self._pref = Pref(self.room, self, self.configDocId, self.path)
        self._viewer = Viewer(self.room, self._listChanged, self.path)

        self.clear()
        self.loadUIFiles()

    def clear(self):
        item = self._layout.takeAt(0)
        while item is not None:
            widget = item.widget()

            widgets = widget.findChildren(BaseWidget)
            for w in widgets:
                w.deregister(remove=True)
                # BaseWidget.widgets.remove(w)

            if isinstance (widget, BaseWidget):
                widget.deregister(remove=True)
                # BaseWidget.widgets.remove(widget)

            widget.hide()
            widget.setParent(None)
            widget.deleteLater()
            item = self._layout.takeAt(0)

        self.update()
        self._idx = 0

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        if self._pref != None: self._pref.destroy(remove)
        if self._viewer != None: self._viewer.destroy(remove)
        self._viewer = None
        self._pref = None

    def _listChanged(self, list):
        self._pref.isReady(ignoreVersion=True)

    def load(self):
        if self._loading:
            self._reload = True
            return 

        self._loading = True

        doc = self._pref.document()
        config = self._pref.configDoc()

        if doc is None: 
            self.clear()
            self._checkReload()
            return

        self.prefSelect.clear()
        self._idxToPreference = []
        titles = {}

        for preference in self._sheetPrototypes:
            title = config.get("sheets." + preference + ".title")
            if title is not None:
                titles[title] = preference
            else:
                titles[preference] = preference

        for k, v in sorted(titles.items()):
            self._idxToPreference.append(v)
            self.prefSelect.addItem(k)

        list = doc.get(self.path)
        if list is None: 
            self.clear()
            self._checkReload()
            return

        # Delete all widgets that do not match the doc type
        items = []
        for i in range(self._layout.count()):
            items.append(self._layout.itemAt(i).widget())
            
        n = len(items)

        if abs(n-len(list)) > 4:
            self.clear()
            items = []
            n = 0
        else:
            i = w = 0
            while i < len(list):
                sheet = list[i]
                if '_sheet' not in sheet: 
                    continue

                if w < len(items):
                    item = items[w]
                    if item.type() != sheet['_sheet']:
                        # print "Delete", item.type(), w
                        del items[w]
                        self.remove(w, changeDoc=False)
                        n-=1
                        i-=1
                    else:
                        w+=1
                i+=1

            cnt = len(list)

            while len(items) > cnt:
                del items[cnt]
                n-=1
                self.remove(cnt, changeDoc=False)

        from twisted.internet import reactor

        # Lock all existing sheets
        for i in range(0, n):
            items[i].setIndex(i)
            items[i].lock()

        # print "creating", (len(list)-n), "sheets"

        # Create all new sheets
        for i in range(n,len(list)):
            sheet = list[i]
            if "_sheet" not in sheet: continue
            # print "CreateSheet", i
            self.createSheet(sheet["_sheet"])

        # Only configure if new sheets available
        if len(list) > n:
            self._configured(None)
        else:
            self._loading = False

    def _configured(self, ignoredparam):
        self._pref.updateAll()
        self._checkReload()

    def _checkReload(self):
        self._loading = False

        if self._reload:
            self._reload = False
            self.load()
    
    def loadUIFiles(self):
        self._sheetPrototypes = {}

        prefs = []

        import wallaby.frontends as frontends
        from twisted.plugin import getCache

        for p in FX.packagePath(FX.appModule + ".prefs"): prefs.append(p)
        for frontend in getCache(frontends):
            for p in FX.packagePath("wallaby.frontends." + frontend + ".prefs"): prefs.append(p)

        import os
        for path in prefs:
            root = re.sub(r'[/\\]','.', path).replace('..', '')
            root = re.sub(r'^.*wallaby\.', 'wallaby.', root)
            files = os.listdir(path)
 
            for file in files:
                if '.py' in file and '.pyc' not in file and 'UI_' in file:
                    basename, ext = os.path.splitext(file)
                    if ext == '.py':
                        mod = FX.imp(root + '.' + basename)
                        print "Loading prefs sheet", root + '.' + basename, mod
                        if mod == None: continue
   
                        import string 
                        cls = basename[0] + string.lower(basename[1]) + basename[2] + string.upper(basename[3]) + basename[4:]
    
                        sheetName = unicode(basename.replace('UI_', ''))
    
                        if not cls in mod.__dict__: continue
                        self._sheetPrototypes[sheetName] = mod.__dict__[cls]
    
    def remove(self, idx, changeDoc=True):
        item = self._layout.takeAt(idx)
        if item is not None:
            for i in range(idx, self._layout.count()):
                self._layout.itemAt(i).widget().decrementIndex()
    
            widget = item.widget()
            widgets = widget.findChildren(BaseWidget)
            for w in widgets:
                w.deregister(remove=True)
                # BaseWidget.widgets.remove(w)
    
            if isinstance (widget, BaseWidget):
                widget.deregister(remove=True)
                # BaseWidget.widgets.remove(widget)

            widget.hide()
            widget.setParent(None)
            widget.deleteLater()
    
            if changeDoc:
                doc = self._pref.document()
                if doc != None: 
                    list = doc.get(self.path) 
                    if list is not None and len(list) > idx:
                        del list[idx]

                    self._pref._throw(Editor.Out.FieldChanged, self.path + ".")
    
            self._idx -= 1
 
        from twisted.internet import reactor
        reactor.callLater(0, self.checkEnableUpDowns)
    
    def createSheet(self, name, locked=True, scrollTo=False):
        if not self._pref: return
    
        if self._pref.document() == None or self._pref.configDoc() == None: 
            self._pref.noDocument()
            return
    
        if name not in self._sheetPrototypes:
            self._pref.notFound(name)
            return
    
        doc = self._pref.document()
        configDoc = self._pref.configDoc()
    
        proto = configDoc.get("sheets."+name+".proto")
        if proto is None:
            proto = {}  
    
        title = configDoc.get("sheets."+name+".title")
        if title is None:
            title = name  
    
        prefix = self.path
    
        path = prefix + "." + unicode(self._idx)
        if doc.get(prefix) == None:
            doc.set(prefix, [])
    
        # PRINT DATA
        #print "DATA ", doc._data, prefix
    
        list = doc.get(prefix)
        if self._idx > len(list)-1:
            list.append(copy.deepcopy(proto))
        else:
            doc.merge(path, proto)
    
        #print " ARUMENTS ", path, name, doc._data
        doc.set(path + "._sheet", name) 

        # print "Creating sheet", path
    
        sheet = PrefSheet(name, self.room, prefix, self, self._idx, title, locked=locked)
        self._idx += 1
        
        sheet.ui = self._sheetPrototypes[name]()
        sheet.ui.setupUi(sheet.content)
    
        self._layout.addWidget(sheet)

        self.checkEnableUpDowns()
    
        self._pref._throw(Editor.Out.FieldChanged, path + ".")
    
        if scrollTo:
            self.scrollArea.ensureWidgetVisible(sheet, 50, 100)
    
        self.update()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import json
from wallaby.common.draggable import *
from dragLogic import *

class DragPushButton(QtGui.QPushButton, DragLogic):

    def __init__(self, *args):
        QtGui.QPushButton.__init__(self, *args)
        DragLogic.__init__(self)

    def mousePressEvent(self, event):
        DragLogic.mousePressEvent(self, event)

    def mouseMoveEvent(self, event):
        DragLogic.mouseMoveEvent(self, event)



#!/usr/bin/env python
# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import os, re, shutil, os.path, platform, sys, glob

from wallaby.qt_combat import *

def compile(path):
    #platform
    uname     = platform.uname()
    
    if re.search("2.6", sys.version):
        pythonpath  = '/System/Library/Frameworks/Python.framework/Versions/2.6/bin/'
    elif re.search("2.7", sys.version):
        # Default install path for python.org python 2.7 dmg on OSX 10.6
        if os.path.isdir("/Library/Frameworks/Python.framework/Versions/2.7/bin/"):
            pythonpath = '/Library/Frameworks/Python.framework/Versions/2.7/bin/'
        else:
            pythonpath  = '/System/Library/Frameworks/Python.framework/Versions/2.7/bin/'
    else:
        print 'No supported version of python found (2.6/2.7) != '+sys.version
        exit(1);
    
    if not USES_PYSIDE:
        uic = 'pyuic4'
        rcc = 'pyrcc4'
    
        if os.path.isfile('/usr/local/bin/pyrcc4'):
            pythonpath = '/usr/local/bin/'

        if os.path.isfile('/usr/bin/pyrcc4'):
            pythonpath = '/usr/bin/'
    else:
        uic = 'pyside-uic'
        rcc = 'pyside-rcc'
    
        if os.path.isfile('/usr/local/bin/pyside-rcc'):
            pythonpath = '/usr/local/bin/'

        if os.path.isfile('/usr/bin/pyside-rcc'):
            pythonpath = '/usr/bin/'

    if not os.path.exists(pythonpath+uic):
        pythonpath = ''

    for file in glob.glob(path + "/*.ui"):    
        root, file = os.path.split(file)
        basename, ext = os.path.splitext(file)

        src = os.path.join(root, file)
        dst = os.path.join(root, 'UI_' + basename + '.py')

        if not os.path.exists(dst) or os.stat(src).st_mtime > os.stat(dst).st_mtime:
            print "translating", src, "->", dst
    
            #OSX
            if uname[0] in ('Darwin', 'Linux'):
                os.system(pythonpath+uic+' --from-imports '+src+' > '+dst)
            #Windows
            elif uname[0] == 'Windows':
                os.system('pyuic4.bat --from-imports '+src+' > '+dst)

    for file in glob.glob(path + "/*.qrc"):
        root, file = os.path.split(file)
        basename, ext = os.path.splitext(file)
        src = os.path.join(root, file)
        dst = os.path.join(root, basename + '_rc.py')
    
        if not os.path.exists(dst) or os.stat(src).st_mtime > os.stat(dst).st_mtime:
            print "translating", src, "->", dst
    
            #OSX
            if uname[0] in ('Darwin', 'Linux'):
                os.system(pythonpath+rcc+' '+src+' > '+dst)
            #Windows
            elif uname[0] == 'Windows':
                os.system('pyrcc4.exe '+src+' > '+dst)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX
import wallaby.FXUI as FXUI
from functools import partial

from wallaby.qt_combat import *

from wallaby.common.pathHelper import PathHelper
from wallaby.pf.peer.embeddedViewer import *

class EmbeddedViewTableModel(QtCore.QAbstractTableModel):
    def __init__(self, widget, room, path, cols=['_id'], parent=None, conflictCB=None, isList=True, identifier=None, reverseOrder=False, minRowHeight=None, minColumnWidth=None, wrapInList=False, sizeHints=None, editOnInsert=False, *args):
        QtCore.QAbstractTableModel.__init__(self, parent, *args)

        self._peer   = EmbeddedViewer(room, path, delegate=self, conflictCB=conflictCB, isList=isList, wrapInList=wrapInList, identifier=identifier, editOnInsert=editOnInsert)
        self._widget = widget

        self._reverseOrder = reverseOrder

        self._imageCache = {}

        self._minRowHeight = 0
        self._minColumnWidth = 0
        self._sizeHints = {}

        if minRowHeight != None: self._minRowHeight = int(float(minRowHeight))
        if minColumnWidth != None: self._minColumnWidth = int(float(minColumnWidth))
        if sizeHints != None: self._sizeHints = sizeHints

        self._isList   = isList
        self._wrapInList = wrapInList
        self._columns  = []
        self._labels   = []
        self._types    = []
        self._typeArgs = []
        self._data     = self._emptyData()
        self._parent   = QtCore.QModelIndex()
        self._keys = None
        self._rows = None

        self._length = 0
           
        if isinstance(cols, list) and len(cols) > 0 and isinstance(cols[0], dict):
            self._columns = []
            self._labels = []
            self._types = []
            self._typeArgs = []
            for cfg in cols:
                if cfg is None: continue
                self._columns.append(cfg.get('path', None))
                self._labels.append(cfg.get('label', None))
                self._types.append(FX.convertType(cfg.get('type', None)))
                self._typeArgs.append(cfg.get('type-args', None))
        else: 
            self._columns, self._labels = FX.splitList(cols, ':')
            self._types, self._typeArgs, self._columns = FX.extractType(self._columns)
        self._path = path

    def _emptyData(self):
        if self._isList:
            return []
        else:
            return {}
            

    def resolve(self, **ka):
        if self._peer: self._peer.resolve(**ka)

    def getData(self):
        return self._data

    def destroyPeers(self, remove=False):
        if self._peer != None:
            self._peer.destroy(remove)

        self._peer = None

    def doSelect(self, row):
        self._peer.select(self.getKey(row))

    def editRow(self, key):
        row = self.getRow(key)
        if row != None:
            for col in range(len(self._types)):
                if self._types[col] in ("dictedit", "listedit", "autoedit", "stringedit", "doubleedit", "numberedit", "currencyedit", "comboedit"):
                    self._widget.editCell(row, col)
                    return 

    def selectRow(self, key):
        row = self.getRow(key)
        if row != None:
            self._widget.selectRow(row)

        self._widget.resetSelection()

    def beginInsertCB(self, idx):
        self.beginInsertRows(self._parent, idx, idx)

    def resetSelectionCB(self):
        if self._widget: self._widget.resetSelection()

    def endInsertCB(self, idx):
        if not self._isList:
            self._keys.insert(idx, "__new__")
            self._mapRows()

        self.emit(QtCore.SIGNAL('rowsInserted'), self._parent, idx, idx)
        self.endInsertRows()

    def beginDeleteCB(self, idx):
        self.beginRemoveRows(self._parent, idx, idx)

    def endDeleteCB(self, idx):
        if not self._isList:
            del self._keys[idx]
            self._mapRows()

        self.emit(QtCore.SIGNAL('rowsRemoved'), self._parent, idx, idx)
        self.endRemoveRows()

    def changedCB(self):
        idx = self._widget.selectedIndexes()

        # clear cache
        self._imageCache = {}

        if len(idx) > 1:
            self.dataChanged.emit(idx[0], idx[1])
        elif len(idx) > 0:
            self.dataChanged.emit(idx[0], idx[0])

    def reorderRows(self, row, oldIndex, newIndex):
        self.beginRemoveRows(self._parent, oldIndex, oldIndex)
        item = self._data.pop(oldIndex)
        self.endRemoveRows()

        self.beginInsertRows(self._parent, newIndex, newIndex)
        self._data.insert(newIndex, item)
        self.endInsertRows()

        self._peer.dataChanged(newIndex)

    def dataCB(self, data):
        # import traceback
        # traceback.print_stack()
        if data == None:
            data = self._emptyData()

        cnt = self.rowCount(self._parent)

        self.beginRemoveRows(self._parent, 0, cnt-1)
        self._data = self._emptyData()
        self._imageCache = {}

        self.emit(QtCore.SIGNAL('rowsRemoved'), self._parent, 0, cnt-1)
        self.endRemoveRows()

        self._length = len(data)

        if self._length > 0:
            self.beginInsertRows(self._parent, 0, self._length-1)
            self._data = data

            if not self._isList:
                self._keys = sorted(self._data.keys())
                self._mapRows()

                self.emit(QtCore.SIGNAL('rowsInserted'), self._parent, 0, self._length-1)
            else:
                self.emit(QtCore.SIGNAL('rowsInserted'), self._parent, 0, self._length-1)

            self.endInsertRows()
        else:
            self._keys = []

        self._widget.newData()

    def _mapRows(self):
        i = 0
        self._rows = {}
        for k in self._keys:
            self._rows[k] = i
            i += 1
 

    def _splitList(self, lst, sep):
        firstList = []
        secondList = []

        for e in lst:
            t = e.split(sep)
            first = t[0]
            if len(t) > 1:
                second = t[1]
            else:
                second = first

            firstList.append(first)
            secondList.append(second)

        return firstList, secondList

    def hasChildren(self, index):
        return False

    def parent(self, index):
        return self._parent

    def index(self, row, col, parent=QtCore.QModelIndex()):
        return self.createIndex(row, col)

    def rowCount(self, parent): 
        if self._isList:
            return self._length
        else:
            if self._keys == None:
                return 0
            else:
                return len(self._keys)

    def columnCount(self, parent): 
        return len(self._columns)

    def flags(self, index):
        col = index.column()
        flags = QtCore.QAbstractTableModel.flags(self, index)

        if col < 0 or col >= len(self._types): return flags

        if self._types[col] in ("dictedit", "listedit", "autoedit", "stringedit", "doubleedit", "numberedit", "currencyedit", "comboedit"):
            flags |= QtCore.Qt.ItemIsEditable

        return flags

    def setData(self, index, value, role):
        if role == QtCore.Qt.EditRole:
            col = index.column()
            row = index.row()

            if self.rowCount(self._parent) <= row: return False

            key = self.getKey(row)

            oldVal = None 

            if self._columns[col] == '*':
                if (self._isList and key < len(self._data)) or key in self._data:
                    oldVal = self._data[key]
                else:
                    val = u""
            elif self._columns[col] == '__key__':
                oldVal = key
            else:
                if self._isList:
                    if key < len(self._data): oldVal = PathHelper.getValue(self._data[key], self._columns[col])
                else:
                    if key in self._data: oldVal = PathHelper.getValue(self._data[key], self._columns[col])

            vtype = self._types[col]

            if vtype not in ("autoedit", "dictedit", "listedit") and self._columns[col] == "*" and isinstance(oldVal, (dict, list, tuple)):
                return False

            if vtype == "autoedit":
                if isinstance(oldVal, (float, int)):
                    vtype = "doubleedit"
                elif isinstance(oldVal, bool):
                    vtype = "booledit"
                else:
                    if isinstance(value, (str, unicode)):
                        import re
                        if value == "true" or value == "false":
                            vtype = "booledit"
                        elif re.match(r"^\d+\.?\d*$", value):
                            vtype = "doubleedit"
                        elif re.match(r"^\[", value):
                            vtype = "stringedit"
                            if oldVal is None or not isinstance(oldVal, list): oldVal = []
                        elif re.match(r"^\{", value):
                            vtype = "stringedit"
                            if oldVal is None or not isinstance(oldVal, dict): oldVal = []
                        else:
                            vtype = "stringedit"
                    else:
                        vtype = "stringedit"
            elif vtype == "dictedit":
                vtype = "stringedit"
                if oldVal is None or not isinstance(oldVal, dict): oldVal = {}
            elif vtype == "listedit":
                vtype = "stringedit"
                if oldVal is None or not isinstance(oldVal, list): oldVal = []

            if vtype == "stringedit":
                val = unicode(value)

                if isinstance(oldVal, (list, dict)):
                    import json
                    try:
                        value = json.loads(u'{"value":' + val + u'}')
                        val = value['value']

                    except Exception as e:
                        if isinstance(oldVal, list): val = []
                        elif isinstance(oldVal, dict): val = {}

                if self._columns[col] == '*':
                    self._data[key] = val
                elif not self._isList and self._columns[col] == '__key__':
                    if key in self._data:
                        self._data[val] = self._data[key]
                        del self._data[key]

                    del self._rows[self._keys[row]]
                    self._keys[row] = val
                    self._rows[val] = row
                    self._peer.select(val)
                else:
                    if self._data[key] == None or not isinstance(self._data[key], dict): 
                        self._data[key] = {}

                    PathHelper.setValue(self._data[key], self._columns[col], val)

            elif vtype in ("doubleedit", "numberedit", "currencyedit"):
                val = 0
                if isinstance(value, (unicode, str)):
                    try:
                        val = float(value)
                    except: pass
                elif isinstance(value, (float, int, bool)):
                    val = value

                if self._columns[col] == '*':
                    self._data[key] = val
                elif not self._isList and self._columns[col] == '__key__':
                    self._data[val] = self._data[key]
                    del self._data[key]
                    del self._rows[self._keys[row]]
                    self._keys[row] = val
                    self._rows[val] = row
                    self._peer.select(val)
                else:
                    PathHelper.setValue(self._data[key], self._columns[col], val)
            elif vtype in ("booledit"):
                val = False
                if isinstance(value, (unicode, str)):
                    try:
                        val = bool(value)
                    except: pass
                elif isinstance(value, (bool)):
                    val = value

                if self._columns[col] == '*':
                    self._data[key] = val
                elif not self._isList and self._columns[col] == '__key__':
                    self._data[val] = self._data[key]
                    del self._data[key]
                    del self._rows[self._keys[row]]
                    self._keys[row] = val
                    self._rows[val] = row
                    self._peer.select(val)
                else:
                    PathHelper.setValue(self._data[key], self._columns[col], val)
 

            if vtype in ("stringedit", "doubleedit", "numberedit", "currencyedit", "booledit"):
                self.dataChanged.emit(index, index)
                self._peer.fieldChanged(key, self._columns[col])
                return True

        return False

    def getRow(self, key):
        if self._isList:
            if isinstance(key, (unicode, str)):
                return None
            else:
                return key
        else:
            if self._rows is not None and key not in self._rows:
                return len(self._rows)

            return self._rows[key] 
            
    def getKey(self, row):
        if row == None: return None

        if self._isList:
            return row
        else:
            if row > self._keys:
                return None

            return self._keys[row]

    def data(self, index, role): 
        if not index.isValid(): 
            return None
        elif role != QtCore.Qt.EditRole and role != QtCore.Qt.DisplayRole and role != QtCore.Qt.SizeHintRole:  
            return None

        row = index.row()
        if self.rowCount(self._parent) <= row: return None

        col = index.column()
        if len(self._columns) <= col: return None
 

        if role == QtCore.Qt.SizeHintRole:
            if self._columns[col] in self._sizeHints and isinstance(self._sizeHints[self._columns[col]], dict):
                hint = self._sizeHints[self._columns[col]]
                w = self._minColumnWidth
                h = self._minRowHeight
                if 'width' in hint and hint['width'] is not None: w = int(hint['width'])
                if 'height' in hint and hint['height'] is not None: h = int(hint['height'])
                # print w, h, col
                return QtCore.QSize(w, h)
            else:
                size = QtCore.QSize(self._minColumnWidth, self._minRowHeight)

            # if isinstance(val, QtGui.QPixmap):
            #     size.setWidth(max(val.width()+1, size.width()))
            #     size.setHeight(max(val.height(), size.height()))
            return size

        key = self.getKey(row)

        path = self._columns[col]
        type = self._types[col]
        typeArg = self._typeArgs[col]

        pathes = []
        types = []
        typeArgs = []
        
        if path != None and '|' in path:
            pathes = path.split('|')

            if type != None and '|' in type:
                types = type.split('|')
            else:
                for p in pathes: types.append(type)

            if typeArg != None and '|' in typeArg:
                typeArgs = typeArg.split('|')
            else:
                for p in pathes: typeArgs.append(typeArg)

            type = types[-1]
            for i in range(len(types), len(pathes)):
                types.append(type)

            typeArg = typeArgs[-1]
            for i in range(len(typeArgs), len(pathes)):
                typeArgs.append(typeArg)
        else:
            pathes = [path] 
            types = [type]
            typeArgs = [typeArg]

        i = 0
        for i in range(len(pathes)):
            path = pathes[i]
            type = types[i]
            typeArg = typeArgs[i]

            if path == '*':
                if (self._isList and key < len(self._data)) or key in self._data:
                    val = self._data[key]
                else:
                    val = u""
            elif path == '__key__':
                val = key
            else:
                val = PathHelper.getValue(self._data[key], path)
                # print "get", key, self._data[key], self._columns[col], val

            if type == "imagelist":
                if not isinstance(val, list):
                    continue

                images = []

                for dct in val:
                    if isinstance(dct, (unicode, str)):
                        images.append(dct)
                    elif isinstance(dct, dict):
                        if typeArg in dct:
                            images.append(dct[typeArg])
                    elif isinstance(dct, list):
                        if typeArg != None and int(typeArg) < len(dct):
                            images.append(dct[int(typeArg)])

                if str(images) in self._imageCache:
                    return self._imageCache[str(images)]

                dList = []

                for img in images:
                    dList.append(self._peer.getImage(img))

                d = defer.DeferredList(dList)
                d.addCallback(partial(self._updateField, row, col, str(images)))
                return  [QtGui.QPixmap()]

            if type == "image":
                if val in self._imageCache:
                    val = self._imageCache[val]
                else:
                    if not self._peer.hasImage(val):
                        val = None
                        continue

                    d = self._peer.getImage(val)
                    d.addCallback(partial(self._updateField, row, col, val))
                    val = QtGui.QPixmap()
            else:
                val = FXUI.renderType(val, type, typeArg, role == QtCore.Qt.EditRole) 

            if val != None:
                return val
        return val

    def _updateField(self, row, col, path, val):
        if isinstance(val, list):
            lst = []
            for success, value in val:
                if success:
                    lst.append(value)
        else:
            lst = [val]

        pixmaps = []

        for val in lst:
            img = QtGui.QImage.fromData(val)
            if img.width() > 0:
                origPath = self._columns[col]
                h = 128
                w = 128
                if origPath in self._sizeHints:
                    hint = self._sizeHints[origPath]
                    if 'width' in hint and hint['width'] is not None: w = int(hint['width'])
                    if 'height' in hint and hint['height'] is not None: h = int(hint['height'])

                # print "Scale", row, col, w, h
                img = img.scaled(w, h, QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)

            p = QtGui.QPixmap()
            if img != None:
                p.convertFromImage(img)

            pixmaps.append(p)

        # print "Caching", row, col, path
        self._imageCache[path] = pixmaps
        self.dataChanged.emit(self.index(row, col), self.index(row, col))
        return val

    def headerData(self, col, orientation, role):
        if not QtCore: return
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            if col >= len(self._labels):
                return None

            return unicode(self._labels[col])

        if orientation == QtCore.Qt.Vertical and role == QtCore.Qt.DisplayRole:
            return unicode('%03d' % (col+1))

        return None



# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

"""
This module provides support for Twisted to be driven by the Qt mainloop.

In order to use this support, simply do the following::
    |  app = QApplication(sys.argv) # your code to init Qt
    |  import qt4reactor
    |  qt4reactor.install()

Alternatively:

    |  from twisted.application import reactors
    |  reactors.installReactor('qt4')

Then use L{twisted.internet} APIs as usual.  The other methods here are not
intended to be called directly.

If you don't instantiate a `QApplication` or `QCoreApplication` prior to
installing the reactor, a `QCoreApplication` will be constructed
by the reactor.  `QCoreApplication` does not require a GUI so trial testing
can occur normally.

Twisted can be initialized after `QApplication.exec_()` with a call to
`reactor.runReturn()`. Calling `reactor.stop()` will unhook Twisted but
leaves your Qt application running.
"""


import sys
import gc
from zope.interface import implements
from twisted.internet.interfaces import IReactorFDSet
from twisted.python import log
from twisted.internet import posixbase

from PySide.QtCore import QSocketNotifier, QObject, SIGNAL, QTimer, QCoreApplication
from PySide.QtCore import QEventLoop


class TwistedSocketNotifier(QObject):
    """
    Connection between an fd event and reader/writer callbacks.
    """

    def __init__(self, parent, reactor, watcher, socketType):
        QObject.__init__(self, parent)
        self.reactor = reactor
        self.watcher = watcher
        fd = watcher.fileno()
        self.notifier = QSocketNotifier(fd, socketType, parent)
        self.notifier.setEnabled(True)
        if socketType == QSocketNotifier.Read:
            self.fn = self.read
        else:
            self.fn = self.write
        self.connect(self.notifier, SIGNAL("activated(int)"), self.fn)


    def shutdown(self):
        self.notifier.setEnabled(False)
        self.disconnect(self.notifier, SIGNAL("activated(int)"), self.fn)
        self.fn = self.watcher = None
        self.notifier.deleteLater()
        self.deleteLater()


    def read(self, fd):
        if not self.watcher:
            return
        w = self.watcher
        # doRead can cause self.shutdown to be called so keep a reference to self.watcher
        def _read():
            #Don't call me again, until the data has been read
            self.notifier.setEnabled(False)
            why = None
            try:
                why = w.doRead()
                inRead = True
            except:
                inRead = False
                log.err()
                why = sys.exc_info()[1]
            if why:
                self.reactor._disconnectSelectable(w, why, inRead)
            elif self.watcher:
                self.notifier.setEnabled(True) # Re enable notification following sucessfull read
            self.reactor._iterate(fromqt=True)
        log.callWithLogger(w, _read)


    def write(self, sock):
        if not self.watcher:
            return
        w = self.watcher
        def _write():
            why = None
            self.notifier.setEnabled(False)

            try:
                why = w.doWrite()
            except:
                log.err()
                why = sys.exc_info()[1]
            if why:
                self.reactor._disconnectSelectable(w, why, False)
            elif self.watcher:
                self.notifier.setEnabled(True)
            self.reactor._iterate(fromqt=True)
        log.callWithLogger(w, _write)



class QtReactor(posixbase.PosixReactorBase):
    implements(IReactorFDSet)


    def __init__(self):
        self._reads = {}
        self._writes = {}
        self._notifiers = {}
        self._timer = QTimer()
        self._timer.setSingleShot(True)
        QObject.connect(self._timer, SIGNAL("timeout()"), self.iterate)
        
        self.qApp = QCoreApplication.instance()
        self._ownApp=False
        if self.qApp is None:
            self.qApp=QCoreApplication([])
            self._ownApp=True
        self._blockApp = None
        posixbase.PosixReactorBase.__init__(self)


    def _add(self, xer, primary, type):
        """
        Private method for adding a descriptor from the event loop.

        It takes care of adding it if  new or modifying it if already added
        for another state (read -> read/write for example).
        """
        if xer not in primary:
            primary[xer] = TwistedSocketNotifier(None, self, xer, type)


    def addReader(self, reader):
        """
        Add a FileDescriptor for notification of data available to read.
        """
        self._add(reader, self._reads, QSocketNotifier.Read)


    def addWriter(self, writer):
        """
        Add a FileDescriptor for notification of data available to write.
        """
        self._add(writer, self._writes, QSocketNotifier.Write)


    def _remove(self, xer, primary):
        """
        Private method for removing a descriptor from the event loop.

        It does the inverse job of _add, and also add a check in case of the fd
        has gone away.
        """
        if xer in primary:
            notifier = primary.pop(xer)
            notifier.shutdown()


    def removeReader(self, reader):
        """
        Remove a Selectable for notification of data available to read.
        """
        self._remove(reader, self._reads)


    def removeWriter(self, writer):
        """
        Remove a Selectable for notification of data available to write.
        """
        self._remove(writer, self._writes)


    def removeAll(self):
        """
        Remove all selectables, and return a list of them.
        """
        rv = self._removeAll(self._reads, self._writes)
        return rv


    def getReaders(self):
        return self._reads.keys()


    def getWriters(self):
        return self._writes.keys()


    def callLater(self,howlong, *args, **kargs):
        rval = super(QtReactor,self).callLater(howlong, *args, **kargs)
        self.reactorInvocation()
        return rval


    def reactorInvocation(self):
        self._timer.stop()
        self._timer.setInterval(0)
        self._timer.start()


    def _iterate(self, delay=None, fromqt=False):
        """
        See twisted.internet.interfaces.IReactorCore.iterate.
        """
        self.runUntilCurrent()
        self.doIteration(delay, fromqt)


    iterate = _iterate


    def doIteration(self, delay=None, fromqt=False):
        """
        This method is called by a Qt timer or by network activity on
        a file descriptor.

        If called becuase of network activiy then control should not
        be handed back to Qt as this would cause recursion.
        """

        if not self.running and self._blockApp:
            self._blockApp.quit()

        self._timer.stop()
        delay = max(delay, 1)
        if not fromqt:
            self.qApp.processEvents(QEventLoop.AllEvents, delay * 1000)
        if self.timeout() is None:
            timeout = 0.1
        elif self.timeout() == 0:
            timeout = 0
        else:
            timeout = self.timeout()
        self._timer.setInterval(timeout * 1000)
        self._timer.start()


    def runReturn(self, installSignalHandlers=True):
        self.startRunning(installSignalHandlers=installSignalHandlers)
        self.reactorInvocation()


    def crash(self):
        log.msg('crash')
        super(QtReactor, self).crash()
        if self._blockApp:
            self._blockApp.quit()


    def run(self, installSignalHandlers=True):
        if self._ownApp:
            self._blockApp = self.qApp
        else:
            self._blockApp = QEventLoop()
        self.runReturn()
        self._blockApp.exec_()



def install():
    """
    Install the Qt reactor.
    """
    p = QtReactor()
    from twisted.internet.main import installReactor
    installReactor(p)

__all__ = ["install"]




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class RadioButton(QtGui.QRadioButton, BaseWidget, EnableLogic, ViewLogic, EditLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    value = Meta.property("string")

    triggers = Meta.property("list", default=["","clicked", "pressed", "released", "checked", "unchecked"])

    def __init__(self, *args):
        QtGui.QRadioButton.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QCheckBox, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self.viewCallback)
        EditLogic.__init__(self, Editor, self.editCallback)
        TriggeredPillowsLogic.__init__(self)

        self.clicked.connect(self._clicked)

        from functools import partial
        self.clicked.connect(partial(self.trigger, "clicked", None)) # ignore argument
        self.pressed.connect(partial(self.trigger, "pressed"))
        self.released.connect(partial(self.trigger, "released"))

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

    def _clicked(self, checked):
        if checked:
            self.trigger("checked") 
            if self._editor: self._editor._fieldChanged()
        else:
            self.trigger("unchecked") 

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)
        TriggeredPillowsLogic.register(self)

    def _resolve(self, value, **ka):
        self.viewCallback(value)
        if self._editor: 
            self._editor._resolve(**ka)
            self._editor._fieldChanged()

    def viewCallback(self, value):
        if self.path and value == self.value: 
            self.trigger("checked") 
            self.setChecked(True)

    def editCallback(self):
        return self.value



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX
import wallaby.FXUI as FXUI

from wallaby.qt_combat import *

import sys, re, copy
from wallaby.frontends.qt.meta import *
from wallaby.pf.room import House
from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class BaseWidget(object):
    widgets = []

    def __init__(self, baseType, *params):
        self.__baseType = baseType

        self._config = {}
        self.resetWallabyTemplate()

        for key in self.wallabyProperties.keys():
            self.resetConfig(key)

        self._configDoc = None
        self.wallabyType = self.__class__.__name__

        self._registered    = False
        self._hasConflict   = False
        self._currentValue  = None
        self._conflictValues = None

        self._configViewerRegistered = False

        self._overlay = None
        self._lastConfig = None

        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self._showConflictContextMenu)

        self._template = None
        self._ignorePalette = False

    def registerPalette(self):
        palette = self.childPalette()
        self._origPalette = palette.resolve(palette)
        self._disabledSet = set()
  
        brush = QtGui.QBrush(QtGui.QColor(0, 34, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        self.setAllStateBrush(palette, QtGui.QPalette.ButtonText, brush)
        self.setAllStateBrush(palette, QtGui.QPalette.Text, brush)
        self._pendingPalette = palette.resolve(palette)
      
        brush.setColor(QtGui.QColor(255, 0, 0)) 
        self.setAllStateBrush(palette, QtGui.QPalette.ButtonText, brush)
        self.setAllStateBrush(palette, QtGui.QPalette.Text, brush)
        self.setAllStateBrush(palette, QtGui.QPalette.WindowText, brush)
        self._faultyPalette = palette.resolve(palette)

        self._noConflictPalette = self._origPalette

    def registerConfigViewer(self):
        if not self._ignorePalette:
            self.registerPalette()
        # if not FX.app: return

        self._template = self.getWallabyTemplate()
        if self._template is None:
            self._template = unicode(self.objectName())

        if not self._configViewerRegistered:
            self._configViewerRegistered = True
            House.get("__CONFIG__").catch(Viewer.In.Refresh, self._configChanged)
            House.get("__CONFIG__").catch(Viewer.In.Document, self._configDocChanged)

    def _configDocChanged(self, pillow, doc):
        self._configDoc = doc
   
        if doc != None and doc.get("widgets." + self._template) == None:
            doc.set("widgets." + self._template, self._config)
            House.get("__CONFIG__").throw(Editor.Out.FieldChanged, "widgets." + self._template)

        self._configChanged(None, "widgets." + self._template)

    def _configChanged(self, pillow, path):
        if not self._configDoc or not Viewer.matchPath("widgets." + self._template, path): return
        config = self._configDoc.get("widgets." + self._template)

        if config is not None:
            import json

            if self._lastConfig == None or json.dumps(self._lastConfig, sort_keys=True) != json.dumps(config, sort_keys=True):
                for k, v in config.items():
                    if k == "wallabyType" or (k in self.wallabyProperties and self.wallabyProperties[k].readOnly): 
                        continue
                    self._config[k] = v

                for k, v in self._config.items():
                    if k == "wallabyType" or (k in self.wallabyProperties and self.wallabyProperties[k].readOnly) or k not in config: 
                        config[k] = v

                self._lastConfig = copy.deepcopy(config)

                if self.room != None and len(self.room) > 0: House.get(self.room) # Implicit room creation...
                self.deregister()
                self.register()

    def createOverlay(self):
        if self._overlay == None: 
            from wallabyOverlay import WallabyOverlay
            self._overlay = WallabyOverlay(FXUI.mainWindow, self)

        self._overlay.hide()
        self.resizeOverlay()
        return self._overlay

    def overlayLabel(self):
        label = []
        try:
            room = self.room
            label.append(room)
        except:
            label.append(None)

        try:
            path = self.path
            if path != None: label.append(path)
        except:
            pass

        return label

    def overlayRect(self):
        r = self.visibleRegion().boundingRect()
        r.setX(0)
        r.setY(0)
        return r

    def resizeOverlay(self):
        if self._overlay != None:
            r = self.overlayRect()
            geo = self._overlay.geometry()

            geo.moveTo(self.mapTo(FXUI.mainWindow, QtCore.QPoint(r.x(), r.y())))
            geo.setWidth(r.width())
            geo.setHeight(r.height()) 

            self._overlay.setBaseGeometry(geo)

    def showOverlay(self):
        if QtGui.QWidget.isVisible(self):
            self.createOverlay().show()

    def hideOverlay(self):  
        if self._overlay:
            self._overlay.hide()

    def destroyOverlay(self):
        if self._overlay:
            self._overlay.hide()
            self._overlay.setParent(None)
            self._overlay.deleteLater()
            self._overlay = None

    def _setOrigPalette(self):
        self._noConflictPalette = self._origPalette
        if self._hasConflict == False:
            self.setPalette(self._noConflictPalette)

    def _setPendingPalette(self):
        self._noConflictPalette = self._pendingPalette
        if self._hasConflict == False:
            self.setPalette(self._noConflictPalette)

    def _setFaultyPalette(self):
        self._noConflictPalette = self._faultyPalette
        if self._hasConflict == False:
            self.setPalette(self._noConflictPalette)

    def getMenuEntries(self):
        return {}

    def _showConflictContextMenu(self, pos):
        globalPos = self.mapToGlobal(pos)

        if not self._hasConflict: 
            menuEntries = self.getMenuEntries()
            if menuEntries == None or len(menuEntries) == 0:
                return

            menu = QtGui.QMenu(self)
            for label, action in menuEntries.items():
                pillow = action
                feathers = None

                if isinstance(action, (list, tuple)): 
                    if len(action) == 2:
                        pillow, feathers = action
                    else:
                        pillow = action.pop(0)

                act = menu.addAction(label)
                act.pillow = pillow
                act.feathers = feathers

            action = menu.exec_(globalPos)
            if action != None:
                House.get(self.room).throw(action.pillow, act.feathers)

            return

        menu = QtGui.QMenu()
        if isinstance(self._currentValue, (list, dict)):
            mineAction = QtGui.QAction(FXUI.mineIcon, "use " + str(len(self._currentValue)) + " Item(s) (mine)", self)
        else:
            mineAction = QtGui.QAction(FXUI.mineIcon, "use >" + unicode(self._currentValue) + "< (mine)", self)

        menu.addAction(mineAction)

        mineAllAction = QtGui.QAction(FXUI.mineIcon, "use mine", self)
        menu.addAction(mineAllAction)

        theirsAction = []
        theirsAllAction = []
        
        for user, c in self._conflictValues:
            if user == None: user = "unknown"

            if isinstance(c, (list, dict)):
                act = QtGui.QAction(FXUI.theirsIcon, "use " + str(len(c)) + " Item(s) ("+user+")", self)
            else:
                act = QtGui.QAction(FXUI.theirsIcon, "use >" + unicode(c) + "< ("+user+")", self)
            theirsAction.append(act)
            menu.addAction(act)

            act = QtGui.QAction(FXUI.theirsIcon, "use theirs ("+user+")", self)
            theirsAllAction.append(act)
            menu.addAction(act)

        act = menu.exec_(globalPos)

        if act != None:
            if act == mineAllAction:
                self._resolve(self._currentValue, mine=True)
            elif act == mineAction:
                self._resolve(self._currentValue)
            else:
                if act in theirsAction:
                    idx = theirsAction.index(act)
                    self._resolve(self._conflictValues[idx][1])
                else:
                    idx = theirsAllAction.index(act)
                    self._resolve(self._conflictValues[idx][1], theirs=idx)

            self._conflict(False, self._currentValue, self._conflictValues)

    def _resolve(self, value):
        pass

    def _conflict(self, hasConflict, currentValue, conflictValues):
        if hasConflict:
            self._hasConflict   = True
            self.setPalette(self._faultyPalette)
        else:
            self._hasConflict = False
            self.setPalette(self._noConflictPalette)

        self._currentValue  = currentValue
        self._conflictValues = conflictValues

    def setAllStateBrush(self, palette, t, brush):
        palette.setBrush(QtGui.QPalette.Active  , t, brush)
        palette.setBrush(QtGui.QPalette.Inactive, t, brush)
        palette.setBrush(QtGui.QPalette.Disabled, t, brush)

    def childPalette(self):
        return self.palette()

    def setChildPalette(self, p):
        FX.trace("DEFAULT SET CHILD PALETTE")
        self.setPalette(p)

    def register(self):
        self._registered = True

    def deregister(self, remove=False):
        pass

    def getConfig(self, key=None, type=None, raw=False):
        key = unicode(key)

        if type == None: 
            if key not in self.wallabyProperties: return None
            type = self.wallabyProperties[key].type

        if self.wallabyProperties[key].readOnly:
            return self.wallabyProperties[key].default

        if type == "string":
            if key not in self._config: 
                return self.wallabyProperties[key].default

            val = self._config[key]

            if val == None:
                pass
            else:
                if val == 'None': val = None

            if not raw and val != None and key in ('room', 'path') and len(val) > 0 and val[0] in ('|', '.'):
                p = self.parentWidget()
                while p != None:
                    if isinstance(p, BaseWidget):
                        prefix = p.getConfig(key, type)
                        if prefix != None:
                            return prefix + val
                        else:
                            return val
                    else:
                        p = p.parentWidget()
                
            return val

        elif type in ("float", "double", "int", "bool"):
            if key not in self._config:
                return self.wallabyProperties[key].default

            if type in ("float", "double"):
                try:
                    return float(self._config[key])
                except:
                    return 0.0
            elif type in ("int"):
                try:
                    return int(float(self._config[key]))
                except:
                    return 0
            elif type in ("bool"):
                try:
                    return bool(int(float(self._config[key])))
                except:
                    return False

        elif type == "dict":
            if key not in self._config: 
                return {}

            val = self._config[key]

            if val == None:
                return {}
            else:
                return copy.deepcopy(val)

        elif type == "list":
            if key not in self._config: 
                return []

            val = self._config[key]

            if val == None:
                return []
            else:
                return copy.deepcopy(val)

    def setConfig(self, val, key=None, type=None):
        key = unicode(key)

        if type == None: 
            if key not in self.wallabyProperties: return
            type = self.wallabyProperties[key].type

        if self.wallabyProperties[key].readOnly: return

        if type == "string":
            if val != None:
                val = str(val)

        elif type == "dict":
            if val != None and not isinstance(val, dict):
                dct = {}
                for e in val:
                    if ':' in e:
                        k, v = unicode(e).split(':')
                        try:
                            jsonString = u'{"value":' + v + u'}' 
                            import json
                            obj = json.loads(jsonString)
                            dct[k] = obj['value']
                        except Exception as ex:
                            print "ERROR - Json parse error (", ex, ")", jsonString

                val = dct

        elif type == "list":
            if val != None and not isinstance(val, list):
                lst = []
                for i in val:
                    lst.append(unicode(i))

                val = lst

        elif type in ("bool", "int", "float", "double"):
            pass

        self._config[key] = val
        if self._configDoc != None:
            config = self._configDoc.get("widgets." + self._template)
            config[key] = val
        
    def resetConfig(self, key=None, type=None):
        key = unicode(key)

        if type == None: type = self.wallabyProperties[key].type
        default = self.wallabyProperties[key].default

        if default is not None:
            self._config[key] = default
        else:
            if type == "string":
                self._config[key] = None
            elif type == "bool":
                self._config[key] = False
            elif type == "int":
                self._config[key] = 0
            elif type in ("float", "double"):
                self._config[key] = 0.0
            elif type == "list":
                self._config[key] = []
            elif type == "dict":
                self._config[key] = {}

    def getWallabyTemplate(self):
        if self._wallabyTemplate == "None" or len(self._wallabyTemplate) == 0: return None
        return unicode(self._wallabyTemplate)

    def setWallabyTemplate(self, d):
        if d == None or len(d) == 0: self._wallabyTemplate = "None"
        else: 
            self._wallabyTemplate = unicode(d)
            self.registerConfigViewer()

    def resetWallabyTemplate(self):
        self._wallabyTemplate = "None"




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

class ComboBoxDelegate(QtGui.QItemDelegate):
    def __init__(self, parent, idx):
        QtGui.QItemDelegate.__init__(self, parent)
    def _commitAndCloseEditor(self):
        editor = self.sender()
        self.commitData.emit(editor)
        self.closeEditor.emit(editor)
        editor.deregister(True)

    def createEditor(self, parent, options, index):
        from wallaby.frontends.qt.widgets.combo.comboBox import ComboBox
        editor = ComboBox(parent)
        editor.activated.connect(self._commitAndCloseEditor)
        print "Create Editor", parent.parent().objectName() + unicode(index.column())
        editor.setObjectName(parent.parent().objectName() + unicode(index.column()))
        return editor

    def setEditorData(self, editor, index):
        print "Set Data"
        pass

    def setModelData(self, editor, model, index):
        print "Set Modeldata"
        pass

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/widgets/prefs/prefSheet.ui'
#
# Created: Mon Nov 26 16:21:03 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_PrefSheet(object):
    def setupUi(self, PrefSheet):
        PrefSheet.setObjectName(_fromUtf8("PrefSheet"))
        PrefSheet.resize(530, 296)
        PrefSheet.setMinimumSize(QtCore.QSize(400, 0))
        PrefSheet.setWindowOpacity(1.0)
        self.verticalLayout_2 = QtGui.QVBoxLayout(PrefSheet)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setMargin(0)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.groupBox = QtGui.QGroupBox(PrefSheet)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(100)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setStyleSheet(_fromUtf8(""))
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.toolbar = QtGui.QWidget(self.groupBox)
        self.toolbar.setObjectName(_fromUtf8("toolbar"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout(self.toolbar)
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setMargin(0)
        self.horizontalLayout_4.setMargin(0)
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.widget = QtGui.QWidget(self.toolbar)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setMinimumSize(QtCore.QSize(0, 50))
        self.widget.setAutoFillBackground(False)
        self.widget.setStyleSheet(_fromUtf8("background: transparent;"))
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.edit = QtGui.QPushButton(self.widget)
        self.edit.setMinimumSize(QtCore.QSize(24, 24))
        self.edit.setMaximumSize(QtCore.QSize(24, 24))
        self.edit.setAutoFillBackground(False)
        self.edit.setStyleSheet(_fromUtf8("QPushButton {\n"
"     border: 2px solid #8f8f91;\n"
"     border-radius: 6px;\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #f6f7fa, stop: 1 #dadbde);\n"
" }\n"
"\n"
" QPushButton:pressed {\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #dadbde, stop: 1 #f6f7fa);\n"
" }\n"
"\n"
" QPushButton:flat {\n"
"     border: none; /* no border for a flat push button */\n"
" }\n"
"\n"
" QPushButton:default {\n"
"     border-color: navy; /* make the default button prominent */\n"
" }"))
        self.edit.setText(_fromUtf8(""))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/lock_locked.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/lock_unlocked.png")), QtGui.QIcon.Normal, QtGui.QIcon.On)
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/lock_unlocked.png")), QtGui.QIcon.Disabled, QtGui.QIcon.On)
        self.edit.setIcon(icon)
        self.edit.setIconSize(QtCore.QSize(16, 16))
        self.edit.setCheckable(True)
        self.edit.setDefault(False)
        self.edit.setFlat(True)
        self.edit.setObjectName(_fromUtf8("edit"))
        self.horizontalLayout_2.addWidget(self.edit)
        self.moveDown = QtGui.QPushButton(self.widget)
        self.moveDown.setMinimumSize(QtCore.QSize(24, 24))
        self.moveDown.setMaximumSize(QtCore.QSize(24, 24))
        self.moveDown.setStyleSheet(_fromUtf8("QPushButton {\n"
"     border: 2px solid #8f8f91;\n"
"     border-radius: 6px;\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #f6f7fa, stop: 1 #dadbde);\n"
" }\n"
"\n"
" QPushButton:pressed {\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #dadbde, stop: 1 #f6f7fa);\n"
" }\n"
"\n"
" QPushButton:flat {\n"
"     border: none; /* no border for a flat push button */\n"
" }\n"
"\n"
" QPushButton:default {\n"
"     border-color: navy; /* make the default button prominent */\n"
" }"))
        self.moveDown.setText(_fromUtf8(""))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/arrow_down.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.moveDown.setIcon(icon1)
        self.moveDown.setFlat(True)
        self.moveDown.setObjectName(_fromUtf8("moveDown"))
        self.horizontalLayout_2.addWidget(self.moveDown)
        self.moveUp = QtGui.QPushButton(self.widget)
        self.moveUp.setMinimumSize(QtCore.QSize(24, 24))
        self.moveUp.setMaximumSize(QtCore.QSize(24, 24))
        self.moveUp.setStyleSheet(_fromUtf8("QPushButton {\n"
"     border: 2px solid #8f8f91;\n"
"     border-radius: 6px;\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #f6f7fa, stop: 1 #dadbde);\n"
" }\n"
"\n"
" QPushButton:pressed {\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #dadbde, stop: 1 #f6f7fa);\n"
" }\n"
"\n"
" QPushButton:flat {\n"
"     border: none; /* no border for a flat push button */\n"
" }\n"
"\n"
" QPushButton:default {\n"
"     border-color: navy; /* make the default button prominent */\n"
" }"))
        self.moveUp.setText(_fromUtf8(""))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/arrow_up.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.moveUp.setIcon(icon2)
        self.moveUp.setFlat(True)
        self.moveUp.setObjectName(_fromUtf8("moveUp"))
        self.horizontalLayout_2.addWidget(self.moveUp)
        self.remove = QtGui.QPushButton(self.widget)
        self.remove.setMinimumSize(QtCore.QSize(24, 24))
        self.remove.setMaximumSize(QtCore.QSize(24, 24))
        self.remove.setStyleSheet(_fromUtf8("QPushButton {\n"
"     border: 2px solid #8f8f91;\n"
"     border-radius: 6px;\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #f6f7fa, stop: 1 #dadbde);\n"
" }\n"
"\n"
" QPushButton:pressed {\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #dadbde, stop: 1 #f6f7fa);\n"
" }\n"
"\n"
" QPushButton:flat {\n"
"     border: none; /* no border for a flat push button */\n"
" }\n"
"\n"
" QPushButton:default {\n"
"     border-color: navy; /* make the default button prominent */\n"
" }"))
        self.remove.setText(_fromUtf8(""))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/delete.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.remove.setIcon(icon3)
        self.remove.setFlat(True)
        self.remove.setObjectName(_fromUtf8("remove"))
        self.horizontalLayout_2.addWidget(self.remove)
        self.dragLabel = DragLabel(self.widget)
        self.dragLabel.setMinimumSize(QtCore.QSize(20, 20))
        self.dragLabel.setMaximumSize(QtCore.QSize(20, 20))
        self.dragLabel.setCursor(QtGui.QCursor(QtCore.Qt.ClosedHandCursor))
        self.dragLabel.setStyleSheet(_fromUtf8("QLabel {\n"
"     border-radius: 6px;\n"
"     background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,\n"
"                                       stop: 0 #f6f7fa, stop: 1 #dadbde);\n"
" }"))
        self.dragLabel.setPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/move.png")))
        self.dragLabel.setScaledContents(True)
        self.dragLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dragLabel.setObjectName(_fromUtf8("dragLabel"))
        self.horizontalLayout_2.addWidget(self.dragLabel)
        self.horizontalLayout_4.addWidget(self.widget)
        self.verticalLayout.addWidget(self.toolbar)
        self.content = QtGui.QWidget(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.content.sizePolicy().hasHeightForWidth())
        self.content.setSizePolicy(sizePolicy)
        self.content.setAutoFillBackground(False)
        self.content.setObjectName(_fromUtf8("content"))
        self.verticalLayout.addWidget(self.content)
        self.verticalLayout_2.addWidget(self.groupBox)

        self.retranslateUi(PrefSheet)
        QtCore.QMetaObject.connectSlotsByName(PrefSheet)

    def retranslateUi(self, PrefSheet):
        PrefSheet.setWindowTitle(QtGui.QApplication.translate("PrefSheet", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("PrefSheet", "GroupBox", None, QtGui.QApplication.UnicodeUTF8))
        self.toolbar.setToolTip(QtGui.QApplication.translate("PrefSheet", "edit", None, QtGui.QApplication.UnicodeUTF8))
        self.moveDown.setToolTip(QtGui.QApplication.translate("PrefSheet", "move down", None, QtGui.QApplication.UnicodeUTF8))
        self.moveUp.setToolTip(QtGui.QApplication.translate("PrefSheet", "move up", None, QtGui.QApplication.UnicodeUTF8))
        self.remove.setToolTip(QtGui.QApplication.translate("PrefSheet", "Remove", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.drag.dragLabel import DragLabel
from . import resource_rc



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class LineEdit(QtGui.QLineEdit, BaseWidget, EnableLogic, ViewLogic, EditLogic, EnterPillowLogic, ContextMenuLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    throwWhileEditing = Meta.property("bool", extended=True, default=True)
    triggers = Meta.property("list", readOnly=True, default=["", "changed","enter","finished","key"])

    def getMenuEntries(self): return ContextMenuLogic.getMenuEntries(self)

    def __init__(self, *args):
        QtGui.QLineEdit.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QLineEdit, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self._setText)
        EditLogic.__init__(self, Editor, self.text)
        EnterPillowLogic.__init__(self, self.returnPressed)
        TriggeredPillowsLogic.__init__(self)

        self._firstAccess = True
        self._readOnly = False

        self._changed = False

        from functools import partial
        self.textEdited.connect(self._textEdited)
        self.editingFinished.connect(self._editingFinished)
        self.returnPressed.connect(partial(self.trigger, "enter", None))

    def keyPressEvent(self, event):
        import wallaby.FXUI as FXUI 
        if self.trigger("key", FXUI.key_event_to_name(event)):
            event.accept()
        else:
            event.ignore()
            QtGui.QLineEdit.keyPressEvent(self, event)        

    def _textEdited(self, *args):
        if self.throwWhileEditing and self._changed:
            self.trigger("changed")
            if self._editor: self._editor._fieldChanged()

        self._changed = True

    def _editingFinished(self, *args):
        self.trigger("finished")

        if self._editor:
            self._editor._fieldChanged()

        self._changed = True

    def _setEnabled(self, enabled):
        if self._firstAccess:
            self._readOnly = self.isReadOnly()
            self._firstAccess = False

        if self._readOnly: return

        if self.hideIfDisabled:
            self.setVisible(enabled)
        else:
            self.setReadOnly(not enabled)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)
        EnterPillowLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self, selectCallback=self._select)
        EnterPillowLogic.register(self)
        TriggeredPillowsLogic.register(self)

    def _resolve(self, value, **ka):
        if not value:
            self._setText("-")
        else:
            self._setText(value)

        if self._editor: self._editor._resolve(**ka)
        self._textEdited()

    def _select(self):
        self.setFocus()

    def _setText(self, val):
        if self.text() != val:
            self._changed = False
            self.setText(val)
            self._changed = True

    def _docCB(self, document):
        print self.objectName()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './wallaby/frontends/qt/isheet/gbPillowAndFeathers.ui'
#
# Created: Mon Dec  3 11:50:55 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(424, 126)
        self.verticalLayout = QtGui.QVBoxLayout(Form)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label = QtGui.QLabel(Form)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        self.widget = QtGui.QWidget(Form)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.gridLayout = QtGui.QGridLayout(self.widget)
        self.gridLayout.setContentsMargins(0, -1, 0, -1)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_2 = QtGui.QLabel(self.widget)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.LineEdit = LineEdit(self.widget)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.gridLayout.addWidget(self.LineEdit, 0, 1, 1, 1)
        self.label_3 = QtGui.QLabel(self.widget)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.LineEdit_2 = LineEdit(self.widget)
        self.LineEdit_2.setObjectName(_fromUtf8("LineEdit_2"))
        self.gridLayout.addWidget(self.LineEdit_2, 1, 1, 1, 1)
        self.verticalLayout.addWidget(self.widget)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "Throw the following pillow with optional \"feathers\"", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Form", "Pillow", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("Form", "inspector-pillow", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Form", "Feathers", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit_2.setProperty("template", QtGui.QApplication.translate("Form", "inspector-feathers", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from setuptools import setup, find_packages
import wallaby.FX as FX
import sys

try:
    import py2exe
except: pass

import wallaby.apps.inspector

packages = find_packages('.')
includes = ['twisted', 'pkgutil', 'sip', 'json', 'PyQt4.QtCore', 'PyQt4.QtGui']
#wallabyPackages, _ = FX.wallabyPackages([])
wallabyPackages = ["wallaby", "zope.interface"]

VERSION = '0.1.29'
LICENSE = 'GNU Lesser General Public License (LGPL)'

APP = ['app.py']


PY2EXE_OPTIONS = {
        'includes': includes,
        'packages': wallabyPackages,
        'dll_excludes': ['MSVCP90.dll'],
        'excludes' :  ['_tkagg','_ps','_fltkagg','Tkinter','Tkconstants',
                      '_agg','_cairo','_gtk','gtkcairo','pydoc','sqlite3',
                      'bsddb','curses','tcl',
                      '_wxagg','_gtagg','_cocoaagg','_wx'],
        }

PY2APP_OPTIONS = {
        'strip': False, 
        'argv_emulation': True, 
        # 'semi_standalone': True, 
        'iconfile': 'wallaby.icns',
        # 'site_packages': True, 
        # 'optimize': 2,
        # 'compressed': False,
        'includes': includes,
        'packages': wallabyPackages,
    'excludes' :  ['_tkagg','_ps','_fltkagg','Tkinter','Tkconstants',
                      '_agg','_cairo','_gtk','gtkcairo','pydoc','sqlite3',
                      'bsddb','curses','tcl',
                      '_wxagg','_gtagg','_cocoaagg','_wx'],
    'plist'    : {  'CFBundleDisplayName': '$appname$',
                    'CFBundleGetInfoString' : 'wallaby, FreshX',
                    'CFBundleIdentifier':'com.freshx.wallaby',
                    'CFBundleShortVersionString':VERSION,
                    'CFBundleVersion': '$appname$ ' + VERSION,
                    'CFBundleIconFile': 'wallaby.icns',
                    'LSMinimumSystemVersion':'10.7',
                    'LSMultipleInstancesProhibited':'false',
                    'NSHumanReadableCopyright':LICENSE
                }}

if len(sys.argv) > 1 and sys.argv[1] not in("copy"):
    setup(name='wallaby-app-$appname$',
      version=VERSION,
      url='http://freshx.de/wallaby/apps/$appname$',
      author='FreshX GbR',
      author_email='wallaby@freshx.de',

      windows=[{'script':'app.py', 'icon_resources':[(0, 'wallaby.ico')]}],
      app=APP,
      license=LICENSE,
 
      packages=packages,
      install_requires=['wallaby-app-inspector', 'wallaby-plugin-pdfgenerator'],
      include_package_data = True,

      options={'py2app': PY2APP_OPTIONS, 'py2exe': PY2EXE_OPTIONS},
      # setup_requires=['macholib', 'modulegraph', 'py2app', 'py2exe'],
  )

if len(sys.argv) > 1 and sys.argv[1] in("py2app", "copy"):
    print '*** Removing Qt debug libs ***'
    import os
    for root, dirs, files in os.walk('./dist'):
        for file in files:
            # if 'debug' in file:
            #     print 'Deleting', file
            #     os.remove(os.path.join(root,file))
            if 'test_' in file:
                print 'Deleting', file
                os.remove(os.path.join(root,file))
            elif '_tests' in file:
                print 'Deleting', file
                os.remove(os.path.join(root,file))
            # elif '.pyc' in file:
            #     print 'Deleting', file
            #     os.remove(os.path.join(root,file))
    
    os.chdir('./dist')
    # os.system(r'macdeployqt $appname$.app -dmg -verbose=0')
    os.system(r'macdeployqt wallaby-app-$appname$.app -verbose=0')
    os.chdir('..')
    
    os.system(r'hdiutil create $appname$-mac-' + VERSION + r'.dmg -volname "wallaby@fx" -fs HFS+ -srcfolder "dist"')
